/// V.5.2
#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <OLED_I2C.h>
#include <RTClib.h>
#include <avr/pgmspace.h>

RTC_DS1307 rtc; 
  
#define MY_OLED_STRING_HIGH 16//64/4
#define MY_OLED_CHAR_WIDTH 6//128/16
#define MY_OLED_UPDATE_INTERVAL 300
#define OL_DX 16
#define OL_DY 16

#define WSPR_PAUSE 800

OLED  myOLED(SDA, SCL);
extern uint8_t SmallFont[];
int oled_x=0;
unsigned long last_oled_update=0UL;


/* PINs:
 * 0 - COM RX 
 * 1 - COM TX
 * 2 - RX from GPS (obsolete, not for MEGA2560)
 * 3 - PTT out
 * 4,5,6,7,8,9, A0 - used by LCD Keypad Shield (10 - PWM brightnes control - not used) 
 * 11, 10, 12 - (SDATA, SCLK, FSYNC)to AD98xx sintez
 * 13 - TONE on/off out
 * A1 - TONE on/off in manual CW mode (analog 0 is active - connecting to GND enables TONE)
 * A2 - AD98xx flag (if this input is connected to the GND then AD98xx sintez is in use)
 * A3, A4, A5 - free (may be for I2C)
 * 
 * MEGA2560 board:
 * COM0 - USB PC communication 
 * COM1 - free (or using to RX from PC для синтезаторов старого образца, в корпусе)
 * COM2 - GPS
 * COM3 - synthesizer RN3AUS
 */
 
//define BOARD_UNO 1
#define BOARD_MEGA 2560
//define GPS_DEBUG 1 
bool setF(unsigned long code, bool invPh);
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
#define BTN_UP   1
#define BTN_DOWN 2
#define BTN_LEFT 3
#define BTN_RIGHT 4
#define BTN_SELECT 5
#define BTN_NONE 10

#define MODE_CTRL 1
#define MODE_OP32 2
#define MODE_WSPR 3
#define MODE_WSPR15 4
#define MODE_JT9  5
#define MODE_FW   6
#define MODE_FST4 7
#define MODE_QRSS 8
#define MODE_DFCW 9
#define MODE_HELL 10
#define OPERATION_MODES 10
#define MODE_MSG_QRSS 11
#define MODE_MSG_DFCW 12
#define MODE_MSG_HELL 13
#define MODE_MSG_JT9 14
#define MODE_MSG_FST4 15
#define MODE_DOT_QRSS 16
#define MODE_DOT_DFCW 17
#define MODE_DOT_HELL 18
#define MODE_DOT_JT9 19
#define MODE_DOT_FST4 20
#define MODE_DOT_PAUSE 21
#define MODE_T_BEACON_START 22 
#define MODE_T_BEACON_STOP 23
#define MODE_TIME 24
#define MODE_BEACON 25
#define MODE_SUBMENU 26
#define MODE_CALL 27
#define MODE_QTH 28
#define MODE_PWR 29
#define MODE_GEN 30
#define MODE_LF2 31
#define MODE_BORDERS_LF 32
#define MODE_TIMER_CORRECTION 33
#define MODE_GPS 34
#define MODE_EXTERNAL_GPS 35 
#define MODE_EXTERNAL_SYNT 36
#define MODE_LOAD_FST4 37
#define MODE_CLR_EEPROM 38
#define MODE_MAX 39
#define MODE_TX MODE_MAX+1
#define MODE_WAIT MODE_MAX+2
#define MODE_T60 MODE_MAX+3
#define MODE_T30 MODE_MAX+4
#define MODE_TSTART MODE_MAX+5
#define MODE_PAUSE MODE_MAX+6

bool submenu;

#define MSG_LEN_MAX 16
#define M_LEN_MAX 5
#define CALL_LEN_MAX 6
#define CC_LEN_MAX 5
#define QTH_LEN_MAX 4
#define QQ_LEN_MAX 5
#define MSG_FST4_LEN_MAX 160

char mode=MODE_CTRL, mode_old=0, mode_next=0;
char mode_start;
char msgq[MSG_LEN_MAX+2]; 
char msgd[MSG_LEN_MAX+2];
char msgjt[MSG_LEN_MAX+2];
char msghl[MSG_LEN_MAX+2];
char msgfst4[MSG_FST4_LEN_MAX+2];
byte msgq_pos=0, msgd_pos=0, msgjt_pos=0, msghl_pos=0, msgfst4_pos=0;
char m[M_LEN_MAX+2];
char m_pos=0;

char call[CALL_LEN_MAX+2];
char call_pos=0;
char cc[CC_LEN_MAX+2];
char cc_pos=0;

char qth[QTH_LEN_MAX+2];
char qth_pos=0;
char qq[QQ_LEN_MAX+2];
char qq_pos=0;
byte clr_eeprom=0;

byte list_n=0;
int pwr;
const byte pwr_list[19]={0,3,7,10,13,17,20,23,27,30,33,37,40,43,47,50,53,57,60};
char n_pwr;
const byte jt9_list[5]={1,2,5,10,30};
char cnt_dot_jt9;
const int fst4_list[7]={15,30,60,120,300,900,1800};
char cnt_dot_fst4;


unsigned int updwcnt;
double freq;
byte btn_old=0, last_btn=0;

unsigned long Code, Code_old;
unsigned long CodeQRSS, CodeDFCW, CodeWSPR, CodeWSPR15, CodeOP32, CodeJT9, CodeHELL, CodeFST4, CodeFW, CodeCW;
double freqQRSS, freqDFCW, freqWSPR, freqWSPR15, freqOP32, freqJT9, freqHELL, freqFST4, freqFW, freqCW;
unsigned long dc=1UL;
unsigned long Crystal=11UL;
byte Cycles=11;
double Step=1.0;
byte LF2;
byte borders_lf;
bool foundRTC;

#define CRYSTAL_LIST_N 13
const unsigned long Crystal_list[CRYSTAL_LIST_N] ={5000000UL,10000000UL,12000000UL,12800000UL,12800175UL,13000000UL,13500000UL,15000000UL,15600000UL,16000000UL,16384000UL,20000000UL,25000000UL};

byte DDS_VER;
bool DDSAD;
bool BIT32;
bool GMDDS;
byte GMTONE, GMPTT, GMPH;
byte b0_32; 

volatile byte IsToneOn; 
volatile byte IsPttOn;

int dot_qrss;
int dot_dfcw;
int dot_jt9;
int dot_hell;
int dot_fst4;
int dot_pause;

#define TX_LEN_MAX 255
char tx[TX_LEN_MAX+2];
int tx_pos;
int tx_len;
unsigned long timer_interval;
unsigned long time_of_settime;
double timer_correction;// ms per s
unsigned int tx_dc=1UL;

volatile char t_hour, t_min, t_sec;
volatile char t_set_hour, t_set_min, t_set_sec, t_set_hour_old, t_set_min_old, t_set_sec_old;
unsigned long t_dif;
bool timeIsSet=false;

long t_gps_millis, t_gps_millis_old;
long t_pause_millis;


#define BCN_LOAD 0
#define BCN_RUN  1
#define BCN_QRSS 2
#define BCN_DFCW 3
#define BCN_HELL 4
#define BCN_OP32 5
#define BCN_WSPR 6
#define BCN_WSPR15 7
#define BCN_JT9 8
#define BCN_FW2 9
#define BCN_FW5 10
#define BCN_FW15 11
#define BCN_FW30 12
#define BCN_T60 13
#define BCN_T30 14
#define BCN_TSTART 15
#define BCN_NO_TSTART 16
#define BCN_PAUSE 17
#define BCN_RPT 18
#define BCN_CLR 19
#define BCN_SAVE 20
#define BCN_MAX 21

byte BEACON;
#define BCN_LIST_LEN 16
char bcn_list[BCN_LIST_LEN+2];
byte bcn_pos;
byte bcn=1;
byte t_bcn_start_hh=0, t_bcn_start_mm=0;
byte t_bcn_stop_hh=0, t_bcn_stop_mm=0;


////// для набора тонов FST4
#define FST_0 0
#define FST_1 1
#define FST_2 2
#define FST_3 3
#define FST_DEL 4
#define FST_SAVE 5
#define FST_ERASE 6
#define FST_LOAD 7
#define FST_GEN 8
#define FST_MAX 9
byte fst=5;  // 5

#define EEPROM_CALL_ADR 0
#define EEPROM_QTH_ADR      (EEPROM_CALL_ADR+sizeof(call))
#define EEPROM_PWR_ADR      (EEPROM_QTH_ADR+sizeof(qth))
#define EEPROM_CODE_ADR     (EEPROM_PWR_ADR+sizeof(pwr))
#define EEPROM_MSG_DFCW_ADR (EEPROM_CODE_ADR+sizeof(freq))
#define EEPROM_MODE_ADR     (EEPROM_MSG_DFCW_ADR+sizeof(msgd))
#define EEPROM_CRYSTAL_ADR  (EEPROM_MODE_ADR+sizeof(mode))
#define EEPROM_CODEQRSS_ADR (EEPROM_CRYSTAL_ADR+sizeof(Crystal))
#define EEPROM_CODEDFCW_ADR (EEPROM_CODEQRSS_ADR+sizeof(freqQRSS))
#define EEPROM_CODEWSPR_ADR (EEPROM_CODEDFCW_ADR+sizeof(freqDFCW))
#define EEPROM_CODEOP32_ADR (EEPROM_CODEWSPR_ADR+sizeof(freqWSPR))
#define EEPROM_CODEWSPR15_ADR (EEPROM_CODEOP32_ADR+sizeof(freqOP32))
#define EEPROM_DOTQRSS_ADR  (EEPROM_CODEWSPR15_ADR+sizeof(freqWSPR15))
#define EEPROM_DOTDFCW_ADR  (EEPROM_DOTQRSS_ADR+sizeof(dot_qrss))
#define EEPROM_BCNLIST_ADR  (EEPROM_DOTDFCW_ADR+sizeof(dot_dfcw))
#define EEPROM_MSG_QRSS_ADR (EEPROM_BCNLIST_ADR+sizeof(bcn_list))
#define EEPROM_LF2_ADR      (EEPROM_MSG_QRSS_ADR+sizeof(msgq))
#define EEPROM_BORDERS_LF_ADR (EEPROM_LF2_ADR+sizeof(LF2))
#define EEPROM_BEACON_ADR (EEPROM_BORDERS_LF_ADR+sizeof(borders_lf))
#define EEPROM_TIMERCOR_ADR (EEPROM_BEACON_ADR+sizeof(BEACON))
#define EEPROM_MSG_JT9_ADR (EEPROM_TIMERCOR_ADR+sizeof(timer_correction))
#define EEPROM_CODEJT9_ADR (EEPROM_MSG_JT9_ADR+sizeof(msgjt))
#define EEPROM_DOTJT9_ADR  (EEPROM_CODEJT9_ADR+sizeof(freqJT9))
#define EEPROM_MSG_HELL_ADR (EEPROM_DOTJT9_ADR+sizeof(dot_jt9))
#define EEPROM_CODEHELL_ADR (EEPROM_MSG_HELL_ADR+sizeof(msghl))
#define EEPROM_DOTHELL_ADR  (EEPROM_CODEHELL_ADR+sizeof(freqHELL))
#define EEPROM_MSG_FST4_ADR (EEPROM_DOTHELL_ADR+sizeof(dot_hell))
#define EEPROM_CODEFST4_ADR (EEPROM_MSG_FST4_ADR+sizeof(msgfst4))
#define EEPROM_DOTFST4_ADR  (EEPROM_CODEFST4_ADR+sizeof(freqFST4))
#define EEPROM_CODEFW_ADR (EEPROM_DOTFST4_ADR+sizeof(dot_fst4))
#define EEPROM_DOTPAUSE_ADR (EEPROM_CODEFW_ADR+sizeof(freqFW))
#define EEPROM_CODECW_ADR (EEPROM_DOTPAUSE_ADR+sizeof(dot_pause))
#define EEPROM_TBEACONSTARTHH_ADR (EEPROM_CODECW_ADR+sizeof(freqCW))
#define EEPROM_TBEACONSTARTMM_ADR (EEPROM_TBEACONSTARTHH_ADR+sizeof(t_bcn_start_hh))
#define EEPROM_TBEACONSTOPHH_ADR (EEPROM_TBEACONSTARTMM_ADR+sizeof(t_bcn_start_mm))
#define EEPROM_TBEACONSTOPMM_ADR (EEPROM_TBEACONSTOPHH_ADR+sizeof(t_bcn_stop_hh))
#define EEPROM_DATA_END_ADR (EEPROM_TBEACONSTOPMM_ADR+sizeof(t_bcn_stop_mm))

/*
extern int __bss_end;
extern void *__brkval;
 
// Функция, возвращающая количество свободного ОЗУ (RAM)
int memoryFree()
{
   int freeValue;
   if((int)__brkval == 0)
      freeValue = ((int)&freeValue) - ((int)&__bss_end);
   else
      freeValue = ((int)&freeValue) - ((int)__brkval);
   return freeValue;
}
*/


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////// F S T - 4 W /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Local functions --------------------------------------------------------------------------------
//static char chr_norm_fst4w(char, int);
//static unsigned long fst4w_code_call(char *);
//static unsigned long fst4w_code_grid(char *);
//static void fst4w_code_50_bits(unsigned long, unsigned long, int, int *);
//static void add_crc24(int *);
//static void ldcp_240_74(int *, int *);
//static void transmission_symbols(int *);


// External variables -----------------------------------------------------------------------------
//int fst4w_shift[4];            // TX FST4W shift for DDS

// Local variables --------------------------------------------------------------------------------
const static char matrix[166][20] PROGMEM = { // 240-74 LDCP matrix
  "de8b3201e3c59f55a14",
    "2e06d352ebc5b74c4fc",
    "2e16d6cf5a725c3244c",
    "84f5587edca6d777de4",
    "e152b1e2b5965093ecc",
    "244b4828a2ccf2b5f58",
    "5fbbaade810e123c730",
    "6b7e92a99a918df3d44",
    "bbcec6a63ab757a7278",
    "f5f3f0b89a21ceccdb0",
    "a248c5f1ec2bc816290",
    "c84bbad839a5fe76d0c",
    "ad724129bbf4c7f4570",
    "91adb56e7623a2575cc",
    "cbe995bdf156df2c9e4",
    "92ff6ea492c08c150e0",
    "c4ddbe5a02f6a933384",
    "d2e9befc131dc483858",
    "68567543d1eebcb080c",
    "21fa61d559f9baf6abc",
    "911c4fbbafc72e3db28",
    "7c0b534af4b7d583d50",
    "12ce371b90ee9dfe72c",
    "15a604148872e251ec4",
    "3a3c9f3eb0e0f96edc0",
    "705919ffb636f96b390",
    "43daaaa8163d6bc2bd4",
    "96e11ea798b74b10e98",
    "811150609c9dee8230c",
    "be713f85ab34380f4b0",
    "5a02c4abaaccb8f24c4",
    "67bdebb8863d04768cc",
    "5a449cd90c3dbdfe844",
    "9c7a54d1c4ef7418b84",
    "cd82fefaaf9cd28cd8c",
    "ca47e847fabb0054a38",
    "f0b30cef6aab9e37f98",
    "d948d912fbcc1708710",
    "cce1a7b355053d98270",
    "4cf227c225a9063dd48",
    "2db92612e9ba1418e24",
    "3d215c04c762c3d6a28",
    "77de65500b5624ceb0c",
    "fd1a1df99ded2fb9d88",
    "2a19392c71438410fb8",
    "a9b486a9d26ed579754",
    "b698d244ac78d97a498",
    "3d7975b74d727a5e704",
    "38094225a2bce0e1940",
    "3d3e58fae40fac342b0",
    "7732e839a066e337714",
    "69356c082b7753a47b0",
    "3e868a55dc403a802ac",
    "a0157a14a6bf7fdbbcc",
    "1ab628e11a7ab4a7c44",
    "9da3a2247d7449052f4",
    "199a8a7b114816b97f4",
    "b1c5cde2542061704cc",
    "432fa8d3a153eafbdc8",
    "c4ece7e400d8a89c448",
    "316ecf74e4b983f007c",
    "6a14fa8e713bb5e8adc",
    "da4b957ded8374e3640",
    "0a804dba7c7e4533300",
    "52c342ed033f86580e0",
    "1667da8d6fcf4272470",
    "da2f7038d550fa88d8c",
    "685bcbab1d9dd2c2a44",
    "4c93008b3156b3636bc",
    "726998d6327ac797c3c",
    "44ece7e400d8a8dc448",
    "01f9add00dfe823a948",
    "dbb95f5ce9e371ad720",
    "fc746ee5c76827a8728",
    "b25408029506467f4b4",
    "9b5c9219e21126b7cf8",
    "39ae9f48ba9d1a24f04",
    "7de2699623eb507f938",
    "b9c6e903ee91dd32934",
    "397510d2c6cb5e81de8",
    "20157a14aebf7fdbbec",
    "067f76ea5817a465980",
    "9248f3cea0869feb994",
    "23cde2678004ebe5f80",
    "5b81fe6848f58e3cfa8",
    "a9099ace96bff092904",
    "4afa4b0802b33215438",
    "f4f740396b030360858",
    "fc613f77a35ee1163b8",
    "1a4dc27d7e8cc835ff4",
    "e9b056f153b39def7ec",
    "b62eb777a2f953c7efc",
    "388ae4de514b62d238c",
    "891529af40e85317160",
    "474f1afeb724dbd2ba8",
    "11d70880fd88fdd307c",
    "29f26a3acb76e6a517c",
    "df3e902ff9cadcf776c",
    "e3c42da8445965c09f0",
    "ce277a6aeccc316dc58",
    "4d7841fb71543abd9b8",
    "e63230d2d465fb44750",
    "b6e11fa798b74b14e98",
    "05f189d37c5616547b4",
    "ebdb51a81d1e883baa8",
    "bf5bc736663bcd53ae0",
    "2f8d1cc0936142c08fc",
    "436b22fc36d917b6928",
    "044b482822ccf2b5f58",
    "37b2e839a066e3b7714",
    "2a9b4b765c581f0c51c",
    "10a7d44cecf8e6628dc",
    "ad95f02df6d5502dd4c",
    "bbd34f8afd63deaf564",
    "cabddfeb01fce632788",
    "66b57babeedd6124114",
    "7813e0454fbd462be8c",
    "b6105ed6f01ea621d04",
    "9f68bbcec679d1c088c",
    "673da96e414fc7a0f40",
    "5568adb935e11084abc",
    "f6dd308de5e5c4f6fb0",
    "3b49e80d40ae596c7b4",
    "a3cde2478004ebe5f80",
    "dd8e4f309e919d5ed94",
    "5a4020d387757d7bc28",
    "64f9e02ae32362a255c",
    "630d5942d392334b0dc",
    "0bd7e9f4229b2dee210",
    "bca549a9467d3a2550c",
    "2fef7b1f578c5e28d04",
    "f35e0fdda1be4b3b35c",
    "69ed575e7cc537d2394",
    "7dfdcfbfd5ef3093680",
    "b3b2921af97f251d328",
    "5622d0fe90363522364",
    "fcd4fc7fa04a69d2ac4",
    "1119ea451502ed9ab34",
    "970ee777ec969a41754",
    "688d14f8afec76783dc",
    "4d0b8a1028578407420",
    "d3d2138d9fa268da3e8",
    "df1bdbff898e006394c",
    "8ac478a916bb0b77684",
    "93881997428e2c17a94",
    "4aa510e746245e90c08",
    "e00cb8543f85a5d58b8",
    "9100d8eb74031073044",
    "38710e4235bd1e4003c",
    "6aef311cac4c4dccfd4",
    "58430f577f51c36b3e0",
    "12082fa5d4268a95b4c",
    "7a7435a0aca071e64d0",
    "cd8250ebadc95de15b0",
    "debad40c852e99d64dc",
    "4e6caa5e7c86efef748",
    "a5d4cbb97e726e3c580",
    "7e3a0a2c73ef8553640",
    "b60bfc2fd2bd8f530dc",
    "32dbef097a5f84b0318",
    "4cc7c1cf434300be380",
    "896840945be8eabf7f0",
    "36c9b10ec694819a0a0",
    "349f46a799ef95a47c8",
    "9bdcd4ce2563e560b74",
    "b19fcd7111a335c52ec"};
 
static const int syncword1[8] = {0,1,3,2,1,0,2,3};  // predefined synchronization symbols
static const int syncword2[8] = {2,3,1,0,3,2,0,1};  // predefined synchronization symbols
//static int symbol[160];   // final symbols
//static int symbol_idx;    // currently playing FST4W symbol list index


//=================================================================================================
// Normalize characters space 0..9 A..Z in order 0..36
//=================================================================================================
char chr_norm_fst4w(char bc, int type) 
{
  //type = 0 for "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ" and "0123456789"
  //type = 1 for " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  //type = 2 for " ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  //type = 3 for "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

  char cc = 0;
  int offsetdig = 0, offsetlet = 10;

  switch (type)
  {
    case 1:
      offsetdig = 1;
      offsetlet = 11;
    break;

    case 2:
      offsetlet = 1;
    break;

    case 3:
      offsetlet = 0;
    break;
  }

    if (bc >= '0' && bc <= '9') cc = bc - '0' + offsetdig;
    if (bc >= 'A' && bc <= 'Z') cc = bc - 'A' + offsetlet;
    if (bc >= 'a' && bc <= 'z') cc = bc - 'a' + offsetlet;
    if (bc == ' ' ) cc = 0;

    return(cc);
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Coding of callsign
//=================================================================================================
unsigned long fst4w_code_call(char *call)
{
  int i;
  unsigned long code, ntokens = 2063592, max22 = 4194304;

  // the third character must always be a number
    if (chr_norm_fst4w(call[2], 0) > 9) 
    {
    for (i=5; i>0; i--) 
        call[i] = call[i-1];
      call[0] = ' ';
    }

    code = chr_norm_fst4w(call[0], 1);
    code = code * 36 + chr_norm_fst4w(call[1], 0);
    code = code * 10 + chr_norm_fst4w(call[2], 0);
    code = code * 27 + chr_norm_fst4w(call[3], 2);
    code = code * 27 + chr_norm_fst4w(call[4], 2);
    code = code * 27 + chr_norm_fst4w(call[5], 2);

  return code + ntokens + max22;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Coding of grid
//=================================================================================================
unsigned long fst4w_code_grid(char *grid)
{
  unsigned long code;

    // coding of grid
  code = chr_norm_fst4w(grid[0], 3)*18*10*10;
    code = code + chr_norm_fst4w(grid[1], 3)*10*10;
    code = code + chr_norm_fst4w(grid[2], 0)*10;
  code = code + chr_norm_fst4w(grid[3], 0);

  return code;
}
//------------------------------------------------------------------------------------------------


//=================================================================================================
// Code1 (call), code2 (grid) and power level merged into 50 bits code
//=================================================================================================
void fst4w_code_50_bits(unsigned long code1, unsigned long code2, int ndbm, int *message)
{
  int i;
  unsigned long temp;

  // 28 bits callsign
  temp = code1;
  for (i=27; i>=0; i--)
  {
    message[i] = temp & 0x0001;
    temp = temp >> 1;
  }
    // merge 15 bits grid
  temp = code2;
  for (i=42; i>=28; i--)
  {
    message[i] = temp & 0x0001;
    temp = temp >> 1;
  }

  // merge 5 bits ndbm
  temp = 0.3 * ndbm;
  for (i=47; i>=43; i--)
  {
    message[i] = temp & 0x01;
    temp = temp >> 1;
  }

  // the remaining byte-array elements set to 0
  for (i=48; i<74; i++) message[i] = 0;
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Add a 24-bit CRC to the 50 bits callsign grid and power level compressed
//=================================================================================================
void add_crc24(int *message)
{
  int i, xr, crc[24]; 

  // Linear Feedback Shift Register implementation
  // polynomial 0x100065B (1000000000000011001011011) for 24-bit CRC
  for (i=0; i<24; i++)  crc[i] = 0;
  for (i=0; i<50; i++)
    {
    xr = message[i] ^ crc[23];
      crc[23] = crc[22];
        crc[22] = crc[21];
        crc[21] = crc[20];
        crc[20] = crc[19];
        crc[19] = crc[18];
        crc[18] = crc[17];
      crc[17] = crc[16];
        crc[16] = crc[15];
        crc[15] = crc[14];
        crc[14] = crc[13];
        crc[13] = crc[12];
        crc[12] = crc[11];
      crc[11] = crc[10];
        crc[10] = crc[9] ^ xr;
        crc[9] = crc[8] ^ xr;
        crc[8] = crc[7];
        crc[7] = crc[6];
        crc[6] = crc[5] ^ xr;
        crc[5] = crc[4];
        crc[4] = crc[3] ^ xr;
        crc[3] = crc[2] ^ xr;
        crc[2] = crc[1];
        crc[1] = crc[0] ^ xr;
        crc[0] = xr;
  }
   
  // Add CRC 24
  for (i=0; i<24; ++i) message[73-i] = crc[i];
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Forward error correction via a (240,74) LDPC code
//=================================================================================================
void ldcp_240_74(int *message, int *codeword)
{
  char hex[2];
  int i, j, k, str, bmax, col, sum, generator[74], paritychecks[166];

  // create the parity checks
  for (i=0; i<166; i++)
  {
    for (j=0; j<19; j++)
    {
      //memcpy(hex, matrix[i]+j, 1);
      memcpy_P(hex, matrix[i]+j, 1);
      str = (int)strtol(hex, NULL, 16); 
      if (j==18) bmax = 2;
      else bmax = 4;
      for (k=0; k<bmax; k++)
      {
        col = (j*4) + k;
        if (str & (0x0001 << (3-k))) generator[col] = 1;
        else generator[col] = 0;
      }
    }
    sum = 0;
    for (j=0; j<74; j++)
      sum = sum + (message[j] * generator[j]);
    paritychecks[i] = sum % 2;    
  }
  
  // create the codeword
  for (i=0; i<74; i++) codeword[i] = message[i];
  for (i=74; i<240; i++) codeword[i] = paritychecks[i-74];
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Pairs of codeword bits mapping and synchronization symbols
//=================================================================================================
int transmission_symbols(int *codeword)
{
  int i, code, temp[120];

  // Grayscale mapping : 00->0, 01->1, 11->2, 10->3
  for (i=0; i<120; i++)
  {
    code = codeword[(2*i)+1]+ 2*codeword[2*i];
    if (code <= 1) temp[i] = code;
    if (code == 2) temp[i] = 3;
    if (code == 3) temp[i] = 2;
  }

  // interleave synchronization symbols
  for (i=0; i<8; i++) tx[i] = syncword1[i];
  for (i=8; i<38; i++) tx[i] = temp[i-8];
  for (i=38; i<46; i++) tx[i] = syncword2[i-38];
  for (i=46; i<76; i++) tx[i] = temp[i-16];
  for (i=76; i<84; i++) tx[i] = syncword1[i-76];
  for (i=84; i<114; i++) tx[i] = temp[i-24];
  for (i=114; i<122; i++) tx[i] = syncword2[i-114];
  for (i=122; i<152; i++) tx[i] = temp[i-32];
  for (i=152; i<160; i++) tx[i] = syncword1[i-152]; 
  for(i=0;i<160;i++) tx[i]=tx[i] + 0x31;
  tx_len=160;
  return(160);
}
//-------------------------------------------------------------------------------------------------


//=================================================================================================
// Init FST4W beacon
//=================================================================================================
void fst4w_init_beacon(const char *call_in, const char *grid_in, const int ndbm_in)
{
  int ndbm;
  char call[13], grid[7];
  int message[74];    // 74 bits messsage : 50 bits callsign grid and power level compressed and 24-bit CRC
  int codeword[240];    // 240-bit codeword

  strcpy(call, call_in);
  strcpy(grid, grid_in);
  ndbm = ndbm_in;//wspr_ndbm(ndbm_in);

  fst4w_code_50_bits(fst4w_code_call(call), fst4w_code_grid(grid), ndbm, message);
  add_crc24(message);
  ldcp_240_74(message, codeword);
  transmission_symbols(codeword);
  
 // symbol_idx = 0;
  tx_len=160;
  tx_pos=0;
}
//-------------------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*
//// ф-ции для AD9834
#define hibyte(x) (unsigned int)(x >> 14)    // макрос определения старшего разряда
#define lobyte(x) (unsigned int)(x & 0x3FFF) // макрос определения младшего разряда
#define SDATA  11   // данные DDS
#define SCLK   10     // строб DDS
#define FSYNC  12    // обновление данных в DDS
#define AD98XX_FLAG_PIN A2 // аналоговый вход 1 - если он равен 0 (заземлен) - это означает подключение AD9833
#define AD98XX_PTT  3     // управление PTT
#define AD98XX_TONE 13    // управление тоном
//Инициализация пинов
void setup_AD98xx_pins(void)
{
  pinMode(SCLK, OUTPUT);
  pinMode(FSYNC, OUTPUT);
  pinMode(SDATA, OUTPUT);
  pinMode(AD98XX_PTT, OUTPUT);
  pinMode(AD98XX_TONE, OUTPUT);
  digitalWrite(FSYNC,HIGH);
  digitalWrite(SCLK,HIGH); 
  digitalWrite(SDATA,LOW);
}
// Запись 16 бит в DDS. Первым записывается старший бит. 
void dds_write(unsigned int word)
{
   char i;    
   digitalWrite(FSYNC,LOW);  //FSYNC = 0;    
   for(i = 15; i >= 0; i--)     
   {         
     if ((word >> i) & 1)
       digitalWrite(SDATA,HIGH); //SDATA = 1;
     else
       digitalWrite(SDATA,LOW); //SDATA = 0; 
     digitalWrite(SCLK,LOW);    //SCLK = 0;
     delayMicroseconds(10);     
     digitalWrite(SCLK,HIGH);   //SCLK = 1;
   }
  digitalWrite(FSYNC,HIGH);     //FSYNC = 1;
}

//  Запись частоты в DDS.
void dds_set(unsigned long f)
{ 
   dds_write((lobyte(f) | 0x4000));  // Записываем младшие 14 бит добавляя в начало адрес регистра 01 bin
   dds_write((hibyte(f) | 0x4000));  // Записываем старшие 14 бит добавляя в начало адрес регистра 01 bin
   dds_write(0xC000); // информация о фазе
   dds_write(0x2000); // выбор первого регистра частоты, обнуление бита RESET
}
*/
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//For GPS RX
#define GPS_RX_PIN 2 // пин для приема NMEA from GPPS, на нем есть прерывание
#define GPS_1B_DUR 104  // GPS_RX_SPEED - длительность одной посылки в мкс (104 для 9600, 208 для 4800)
#define GPS_LEN 50
#define GPS_BAUD_LIST_N 8
const unsigned int GPS_Baud_list[GPS_BAUD_LIST_N]={1200, 2400, 4800, 9600, 19200, 38800, 57600, 115200};
bool gps_on=false; 
volatile bool gps_ready=false;
volatile bool gps_detected=false;
char gps_s[GPS_LEN+2];
byte gps_cnt;
double lat,lon;

void latlon_to_loc(void)
{
 qth[0]=65 + lon/20;
 qth[1]=65 + lat/10;
 qth[2]=48 + (lon-(qth[0]-65)*20)/2;
 qth[3]=48 + (lat-(qth[1]-65)*10); 
 qth_pos=3;
}

void gps_read()
{ char c=0;
  static char comma;
  static byte comma_pos;
  static bool flag;
  static char s_lat[6], s_lon[7];
  #ifdef BOARD_UNO
   delayMicroseconds(3*GPS_1B_DUR/2);// ждем первый бит
   for(byte i=0; i<8; i++)
   {
    bitWrite(c,i,digitalRead(GPS_RX_PIN));
    delayMicroseconds(GPS_1B_DUR);
   }
   if(digitalRead(GPS_RX_PIN)==LOW){/*gps_detected=false; gps_ready=false;*/ return;} //нет стопового бита - ошибка
  #endif
  
  #ifdef BOARD_MEGA
    if(Serial2.available()) c=Serial2.read(); else return;
  #endif
  
  if(c<0x0a ||c>126) return;

  if(c=='$') {
    gps_cnt=0; gps_detected=true; comma=0; comma_pos=0;
  }
  else{ 
   gps_s[gps_cnt]=c;gps_cnt++;if(gps_cnt>=GPS_LEN)gps_cnt=GPS_LEN-1;
  }
  // найти нужный заголовок и взять время
  if(gps_cnt==12)
  {   
      if(strncmp(gps_s,"GPRMC,",6)==0 || strncmp(gps_s,"GNRMC,",6)==0){
        comma=0; comma_pos=0; flag=true;
      }
      else flag=false;
  }
   //теперь определить время и QTH-loc
   /// $GPRMC,144326.00,A,5107.0017737,N,11402.3291611,W,0.080,323.3,210307,0.0,E,A*20
   /// $GPRMC,123519,A,3840.038,N,00919.000,W,022.4,084.4,230394,003.1,W*6A
   /// $GPRMC,185035.00,A,5501.72403,N,03635.41021,E,0.249,,300520,,,A*77
   ///  01234567890123456
  if(gps_cnt>12 && flag)
  {  if(c==',') {comma++; comma_pos=gps_cnt;}
     switch(comma)
      { case 1: //  поле - признак A/V
              if(gps_cnt-comma_pos==1)
              { if((gps_s[comma_pos]=='A' || gps_s[comma_pos]=='a' /*|| gps_s[comma_pos]=='V' || gps_s[comma_pos]=='v'*/) && isdigit(gps_s[8])&&isdigit(gps_s[9])&&isdigit(gps_s[10])&&isdigit(gps_s[11])){
                 gps_ready=true;
                 t_set_hour=10*(gps_s[6]-48) + (gps_s[7]-48);
                 t_set_min=10*(gps_s[8]-48) + (gps_s[9]-48);
                 t_set_sec=10*(gps_s[10]-48) + (gps_s[11]-48);
                 /// обновляем счет времени и машинных миллисекунд между моментами взятия точного времени - для коррекции dT
                   t_gps_millis=millis();
                   long dt_mmss=(t_set_hour-t_set_hour_old)*3600 + (t_set_min-t_set_min_old)*60 + (t_set_sec-t_set_sec_old);
                   if(dt_mmss<0)dt_mmss+=86400;//dt_mmss+=3600;
                   long dt_millis=t_gps_millis-t_gps_millis_old;
                   if(dt_mmss>600)
                   {static double dT[5];
                    static byte dT_cnt;
                    static bool dT_flag;
                    if(t_gps_millis_old>0){ 
                      dT[dT_cnt]=(double)(dt_mmss*1000.0 - (t_gps_millis-t_gps_millis_old)) /((double)dt_mmss);
                      if(fabs(timer_correction)<0.001){
                        if(fabs(dT[dT_cnt])<20) timer_correction=dT[dT_cnt];
                      }                     
                      dT_cnt++; if(dT_cnt>4) {dT_cnt=0;dT_flag=true;}
                      if(dT_flag){
                        double dT_new=0; byte cc=0;
                        for(byte i=0; i<5; i++) if(fabs(dT[i])<20){dT_new+=dT[i]; cc++;}
                        if(cc>0) {timer_correction=dT_new/cc;
                         EEPROM.put(EEPROM_TIMERCOR_ADR,timer_correction);
                        }
                      }
                    }
                    t_set_hour_old=t_set_hour;t_set_min_old=t_set_min; t_set_sec_old=t_set_sec;
                    t_gps_millis_old=t_gps_millis;
                   }
                  //////
                 setmytime(); //устанавливаем, если время правильное
                }
                else {gps_ready=false;comma=6;}
              }
              break;
        case 2: //  поле - широта
              if(gps_cnt-comma_pos==4){s_lat[0]=gps_s[comma_pos];s_lat[1]=gps_s[comma_pos+1];s_lat[2]=gps_s[comma_pos+2];s_lat[3]=gps_s[comma_pos+3];}
              break;
        case 3: //  поле - знак широты
              if(gps_cnt-comma_pos==1)
              {switch(gps_s[comma_pos])
               { case 'N': case 'n': s_lat[4]='N'; break;
                 case 'S': case 's': s_lat[4]='S'; break;
                 default: {gps_ready=false;comma=6;}
               }
              }
              break;
        case 4: //  поле - долгота
              if(gps_cnt-comma_pos==5){s_lon[0]=gps_s[comma_pos];s_lon[1]=gps_s[comma_pos+1];s_lon[2]=gps_s[comma_pos+2];s_lon[3]=gps_s[comma_pos+3];s_lon[4]=gps_s[comma_pos+4];}
              break;
        case 5: //  поле - знак долготы
              if(gps_cnt-comma_pos==1)
              { switch(gps_s[comma_pos])
                { case 'W': case 'w': s_lon[5]='W'; break;
                  case 'E': case 'e': s_lon[5]='E'; break;
                  default: gps_ready=false;
                }
                if(gps_ready){
                  lat=(s_lat[0]-48)*10 + (s_lat[1]-48) + ((s_lat[2]-48)*10 + (s_lat[3]-48))/60.0;
                  if(s_lat[4]=='S')lat=90-lat; else lat=90+lat;
                  lon=(s_lon[0]-48)*100 + (s_lon[1]-48)*10 + (s_lon[2]-48) + ((s_lon[3]-48)*10 + (s_lon[4]-48))/60.0;
                  if(s_lon[5]=='W')lon=180-lon; else lon=180+lon;;
                  latlon_to_loc();
                  flag=false;
                  //установили и выключаем
                  //disable_gps();
                }
                comma++;
              }
              break;
      }
   }
}
void onoff_gps(void){
 gps_ready=false; gps_detected=false; gps_cnt=0; memset(gps_s,0,sizeof(gps_s));
 #ifdef BOARD_MEGA
  while(Serial2.available()) Serial2.read(); 
 #endif
}
void enable_gps(void)
{ if(gps_on) return;
  gps_on=true; 
  onoff_gps();
 #ifdef BOARD_UNO
   pinMode(GPS_RX_PIN,INPUT);
   attachInterrupt(digitalPinToInterrupt(GPS_RX_PIN), gps_read, FALLING);
 #endif
}
void disable_gps(void)
{ if(!gps_on) return;
  onoff_gps();
 #ifdef BOARD_UNO   
  detachInterrupt(digitalPinToInterrupt(GPS_RX_PIN));
 #endif
  gps_on=false;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define LEN_ABC 38
char dfcw1[8], hell1[20];
const static char abc[LEN_ABC+1]={"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ /"};
/*
const static char  *  const dfcw[] PROGMEM={"-----","_----","__---","___--","____-","_____","-____","--___","---__","----_",
               "_-","-___","-_-_","-__","_","__-_","--_","____","__","_---","-_-","_-__","--","-_","---","_--_",
               "--_-","_-_","___","-","__-","___-","_--","-__-","-_--","--__",
               "-___-","-__-_"};
const static char * const hell[] PROGMEM={"427181929497000","1121376789000","173919581000","2815915937000","57950513579000",            //0 1 2 3 4
               "157951951939000","3618691969148000","1395969799000","241751951937000","1752939486000",       //5 6 7 8 9
               "123454749741000","135791591573000","427191919198000","426161613579000","52851951968000",     //A B C D E
               "135795959599000","528191949418000","1357950513579000","31619112000","3219191939579000",      //F G H I J
               "135795361789000","12315171912000","135797535797531000","1357975313579000","316181929497000", //K L M N O
               "13579494957000", "36181919392471000","135795391597000","21861951938000","9195890909000",     //P Q R S T 
               "2519101012469000","996310135799000","975313575313579000","19284653719000","1916243579000",   //U V W X Y 
               "12919495169189000","00000000","123456789000"};                                               //Z SP /

*/

const char hl0[] PROGMEM={"427181929497000"};
const char hl1[] PROGMEM={"1121376789000"};
const char hl2[] PROGMEM={"173919581000"};
const char hl3[] PROGMEM={"2815915937000"};
const char hl4[] PROGMEM={"57950513579000"};
const char hl5[] PROGMEM={"157951951939000"};
const char hl6[] PROGMEM={"3618691969148000"};
const char hl7[] PROGMEM={"1395969799000"};
const char hl8[] PROGMEM={"241751951937000"};
const char hl9[] PROGMEM={"1752939486000"};
const char hlA[] PROGMEM={"123454749741000"};
const char hlB[] PROGMEM={"135791591573000"};
const char hlC[] PROGMEM={"427191919198000"};
const char hlD[] PROGMEM={"426161613579000"};
const char hlE[] PROGMEM={"52851951968000"};
const char hlF[] PROGMEM={"135795959599000"};
const char hlG[] PROGMEM={"528191949418000"};
const char hlH[] PROGMEM={"1357950513579000"};
const char hlI[] PROGMEM={"31619112000"};
const char hlJ[] PROGMEM={"3219191939579000"};
const char hlK[] PROGMEM={"135795361789000"};
const char hlL[] PROGMEM={"12315171912000"};
const char hlM[] PROGMEM={"135797535797531000"};
const char hlN[] PROGMEM={"1357975313579000"};
const char hlO[] PROGMEM={"316181929497000"};
const char hlP[] PROGMEM={"13579494957000"};
const char hlQ[] PROGMEM={"36181919392471000"};
const char hlR[] PROGMEM={"135795391597000"};
const char hlS[] PROGMEM={"21861951938000"};
const char hlT[] PROGMEM={"9195890909000"};
const char hlU[] PROGMEM={"2519101012469000"};
const char hlV[] PROGMEM={"996310135799000"};
const char hlW[] PROGMEM={"975313575313579000"};
const char hlX[] PROGMEM={"19284653719000"};
const char hlY[] PROGMEM={"1916243579000"};
const char hlZ[] PROGMEM={"12919495169189000"};
const char hl_[] PROGMEM={"00000000"};
const char hlSlash[] PROGMEM={"123456789000"};
const char* const hell[] PROGMEM = 
{hl0,hl1,hl2,hl3,hl4,hl5,hl6,hl7,hl8,hl9,hlA,hlB,hlC,hlD,hlE,hlF,hlG,hlH,hlI,hlJ,hlK,hlL,hlM,hlN,hlO,hlP,hlQ,hlR,hlS,hlT,hlU,hlV,hlW,hlX,hlY,hlZ,hl_,hlSlash};


const char cw0[] PROGMEM={"-----"};
const char cw1[] PROGMEM={".----"};
const char cw2[] PROGMEM={"..---"};
const char cw3[] PROGMEM={"...--"};
const char cw4[] PROGMEM={"....-"};
const char cw5[] PROGMEM={"....."};
const char cw6[] PROGMEM={"-...."};
const char cw7[] PROGMEM={"--..."};
const char cw8[] PROGMEM={"---.."};
const char cw9[] PROGMEM={"----."};
const char cwA[] PROGMEM={".-"};
const char cwB[] PROGMEM={"-..."};
const char cwC[] PROGMEM={"-.-."};
const char cwD[] PROGMEM={"-.."};
const char cwE[] PROGMEM={"."};
const char cwF[] PROGMEM={"..-."};
const char cwG[] PROGMEM={"--."};
const char cwH[] PROGMEM={"...."};
const char cwI[] PROGMEM={".."};
const char cwJ[] PROGMEM={".---"};
const char cwK[] PROGMEM={"-.-"};
const char cwL[] PROGMEM={".-.."};
const char cwM[] PROGMEM={"--"};
const char cwN[] PROGMEM={"-."};
const char cwO[] PROGMEM={"---"};
const char cwP[] PROGMEM={".--."};
const char cwQ[] PROGMEM={"--.-"};
const char cwR[] PROGMEM={".-."};
const char cwS[] PROGMEM={"..."};
const char cwT[] PROGMEM={"-"};
const char cwU[] PROGMEM={"..-"};
const char cwV[] PROGMEM={"...-"};
const char cwW[] PROGMEM={".--"};
const char cwX[] PROGMEM={"-..-"};
const char cwY[] PROGMEM={"-.--"};
const char cwZ[] PROGMEM={"--.."};
const char cw_[] PROGMEM={"-...-"};
const char cwSlash[] PROGMEM={"-..-."};
const char* const dfcw[] PROGMEM = 
{cw0,cw1,cw2,cw3,cw4,cw5,cw6,cw7,cw8,cw9,cwA,cwB,cwC,cwD,cwE,cwF,cwG,cwH,cwI,cwJ,cwK,cwL,cwM,cwN,cwO,cwP,cwQ,cwR,cwS,cwT,cwU,cwV,cwW,cwX,cwY,cwZ,cw_,cwSlash};

//----------------------------------------------------------------
char dfcw_to_char(char mm[])
{
 for(int i=0; i<LEN_ABC; i++)
 {
  strcpy_P(dfcw1,(PGM_P)pgm_read_word(&dfcw[i]));  
  if(strcmp(mm, dfcw1)==0) return abc[i]; 
 }
 return 0;
}
//------------------------------------------------------------------
char * char_to_dfcw(char c)
{ 
  for(int i=0; i<LEN_ABC;i++)
  {
    if(c==abc[i]) 
    { strcpy_P(dfcw1,(PGM_P)pgm_read_word(&dfcw[i]));
      return dfcw1;
    }
  }
  return " ";
}
//------------------------------------------------------------------
char * char_to_hell(char c)
{ 
  for(int i=0; i<LEN_ABC;i++)
  {
    if(c==abc[i]){ 
      strcpy_P(hell1,(PGM_P)pgm_read_word(&hell[i]));
      return hell1;
    }
  }
  return " ";
}

void recomputeCode(void)
{
  double half_step=0.5*Step;
  Code=(unsigned long)(freq/Step);         if(freq-Code*Step>=half_step)Code++;
  CodeCW=(unsigned long)(freqCW/Step); if(freqCW-CodeCW*Step>=half_step)CodeCW++;
  CodeQRSS=(unsigned long)(freqQRSS/Step); if(freqQRSS-CodeQRSS*Step>=half_step)CodeQRSS++;
  CodeDFCW=(unsigned long)(freqDFCW/Step); if(freqDFCW-CodeDFCW*Step>=half_step)CodeDFCW++;
  CodeHELL=(unsigned long)(freqHELL/Step); if(freqHELL-CodeHELL*Step>=half_step)CodeHELL++;
  CodeOP32=(unsigned long)(freqOP32/Step); if(freqOP32-CodeOP32*Step>=half_step)CodeOP32++;
  CodeWSPR=(unsigned long)(freqWSPR/Step); if(freqWSPR-CodeWSPR*Step>=half_step)CodeWSPR++;
  CodeWSPR15=(unsigned long)(freqWSPR15/Step); if(freqWSPR15-CodeWSPR15*Step>=half_step)CodeWSPR15++;
  CodeJT9=(unsigned long)(freqJT9/Step); if(freqJT9-CodeJT9*Step>=half_step)CodeJT9++;
  CodeFST4=(unsigned long)(freqFST4/Step); if(freqFST4-CodeFST4*Step>=half_step)CodeFST4++;

}
//-------------------------------------------------------------------
void set28bit(void)
{ 
  Cycles=1;
  Step=Crystal;//Cycles;
  Step=Step/16384UL;
  Step=Step/16384UL;
  recomputeCode();
  BIT32=false;
  GMDDS=false;
  DDS_VER=0;
  DDSAD=true;
}
//---------------------------------------------------------------------------

//-------------------------------------------------------------------
void set32bit(void)
{ 
  //if(Cycles==12) return;
  Cycles=12;
  Step=Crystal/Cycles;
  Step=Step/65536UL;
  Step=Step/65536UL;
  recomputeCode();
  BIT32=true;
  if(GMDDS){Serial3.begin(9600);Serial3.setTimeout(100);}
  GMDDS=false;
  DDSAD=false;
}
//---------------------------------------------------------------------------
void set24bit(void)
{ 
  //if(Cycles==11) return;
  Cycles=11;
  Step=Crystal/Cycles;
  Step=Step/4096;
  Step=Step/4096;
  recomputeCode();
  BIT32=false;
  if(GMDDS){Serial3.begin(9600);Serial3.setTimeout(100);}
  GMDDS=false;
  DDSAD=false;
}
//---------------------------------------------------------------------------
void setgmdds(void)
{ //if(Cycles==10) return;
  Cycles=10; BIT32=true;
  Step=Crystal/Cycles;
  Step=Step/65536UL;
  Step=Step/65536UL;
  DDS_VER=0;
  recomputeCode();
  if(!GMDDS){Serial3.begin(115200);Serial3.setTimeout(100);}
  GMDDS=true;
  DDSAD=false;
}
//---------------------------------------------------------------------------
bool toneON(void)
{ unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
//  digitalWrite(AD98XX_TONE,HIGH);
  IsToneOn=true;
  //if(DDSAD){ dds_write(Code); return true;}
  if(GMDDS)
  { GMTONE=0;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    bool result=false;
    Serial3.write(c,4);
    result=readF();
    return result;
  }
  c[0]='1';
  unsigned long N=1;
  Serial3.write(c,1);
  return true;
}
//---------------------------------------------------------------------------
bool toneOFF(void)
{ unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
  //digitalWrite(AD98XX_TONE,LOW);
  IsToneOn=false;
  //if(DDSAD) {dds_write(0);return true;}
  if(GMDDS)
  { GMTONE=1;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    bool result=false;
    Serial3.write(c,4);
    result=readF();
    return result;
  }
  c[0]='0';
  Serial3.write(c,1);
  return true;
}
//---------------------------------------------------------------------------
bool pttON(void)
{ bool result=false;
  unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
  //digitalWrite(AD98XX_PTT,HIGH);
  IsPttOn=true;

  //if(DDSAD)return true;
  
  if(GMDDS)
  { 
    GMPTT=1;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    Serial3.write(c,4);
    result=readF();
  }
  else{
   c[0]='T';
   Serial3.write(c,1);
   result=readF();
     }
  return result;
}

bool pttOFF(void)
{ bool result=false;
 unsigned char c[5];
 c[1]=c[2]=c[3]=c[4]=0;
 //digitalWrite(AD98XX_PTT,LOW);
 IsPttOn=false;

 //if(DDSAD) return true;
 
  if(GMDDS)
  { GMPTT=0;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    Serial3.write(c,4);
    result=readF();
  }
   else{
    c[0]='R';
    Serial3.write(c,1);
    result=readF();
       }
  return result;
}
//-----------------------------------------------------------------------
bool setF4(unsigned long code[])
{
 unsigned char c[14];
 if(LF2) for(int i=0; i<3; i++) code[i]=code[i]<<1;
   c[0]='L';
   c[1]=0xff & (code[0]>>16);
   c[2]=0xff & (code[0]>>8);
   c[3]=0xff & (code[0]>>0);
   c[4]=0xff & (code[1]>>16);
   c[5]=0xff & (code[1]>>8);
   c[6]=0xff & (code[1]>>0);
   c[7]=0xff & (code[2]>>16);
   c[8]=0xff & (code[2]>>8);
   c[9]=0xff & (code[2]>>0);
   c[10]=0xff & (code[3]>>16);
   c[11]=0xff & (code[3]>>8);
   c[12]=0xff & (code[3]>>0);
   c[13]=0;
   bool result=false;
   for(int i=0; i<13; i++)
   {
     Serial3.write(&c[i],1);
     delayMicroseconds(10000);
   }
   result=readF();
   return result;
}

//-----------------------------------------------------------------------
bool setFastF(byte f)
{
 if(GMDDS) return false;
  unsigned char c=0;
  switch(f)
  { case 0: c=0xf0; break;
    case 1: c=0xf1; break;
    case 2: c=0xf2; break;
    case 3: c=0xf3; break;
    case 4: c=0xf3; break;
  }
  Serial3.write(&c,1);
  return true;
}
//-----------------------------------------------------------------------

int detectButton() {
  static unsigned long tb;
  int keyAnalog =  analogRead(A0);
//Serial.println(keyAnalog);//!!!!!!!!!!!!!!!!!!!!!!!
  if (keyAnalog < 100) {
    // Значение меньше 100 – нажата кнопка right
    return BTN_RIGHT;
  } else if (keyAnalog < 200) {
    // Значение больше 100 (иначе мы бы вошли в предыдущий блок результата сравнения, но меньше 200 – нажата кнопка UP
    return BTN_UP;
  } else if (keyAnalog < 400) {
    // Значение больше 200, но меньше 400 – нажата кнопка DOWN
    return BTN_DOWN;
  } else if (keyAnalog < 600) {
    // Значение больше 400, но меньше 600 – нажата кнопка LEFT
    return BTN_LEFT;
  } else if (keyAnalog < 800) {
    // Значение больше 600, но меньше 800 – нажата кнопка SELECT
    return BTN_SELECT;
  } else {
    // Все остальные значения (до 1023) будут означать, что нажатий не было
    return BTN_NONE;
  }
}

void clearLine(int line){
  lcd.setCursor(0, line);
  lcd.print(F("                "));
  lcd.setCursor(0, line);
  oled_x=0;
  myOLED.print(F("                               "), OL_DX, line * MY_OLED_STRING_HIGH+OL_DY);
  //myOLED.update();
}

void printError(char err[])
{
 clearLine(1); lcd.print(F("ERR:"));lcd.print(err); /*delay(5000);*/
 myOLED.print(F("ERR:"), OL_DX,  MY_OLED_STRING_HIGH+OL_DY);
 myOLED.print(err, 5*MY_OLED_CHAR_WIDTH+OL_DX,  MY_OLED_STRING_HIGH+OL_DY);
 myOLED.update();
}

void printFreq(unsigned long CC){
  
  double F=CC*Step; //Code=CC;
  if(mode>OPERATION_MODES && mode!=MODE_TX)return;
  lcd.setCursor(0,0);lcd.print(F("            "));
  lcd.setCursor(0,0);lcd.print(F);
  myOLED.print(F("            "), OL_DX,  OL_DY);
  //myOLED.setFont(MediumNumbers);
  myOLED.printNumF(F, 2, OL_DX, OL_DY);
  //myOLED.setFont(SmallFont);
  //if(millis()-last_oled_update>500){myOLED.update();last_oled_update=millis();}
  my_oled_update();
  //lcd.print(F("  "));
}
void printTonePtt(void)
{
  clearLine(1);
  if(IsToneOn==1) lcd.print(F("TONE=ON "));else lcd.print(F("TONE=OFF"));
  if(IsToneOn==1) myOLED.print(F("TONE=ON "),OL_DX,MY_OLED_STRING_HIGH+OL_DY);else myOLED.print(F("TONE=OFF"),OL_DX,MY_OLED_STRING_HIGH+OL_DY);

  lcd.setCursor(9,1); 
  if(IsPttOn==1) lcd.print(F("PTT=ON "));else lcd.print(F("PTT=OFF"));
  if(IsPttOn==1) myOLED.print(F("PTT=ON "),9*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);else myOLED.print(F("PTT=OFF"),9*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  
  myOLED.update();
}

void paintTX(void)
{ 
  unsigned int percent=99*tx_pos/tx_len;
  unsigned int time_to_end=(tx_len-tx_pos)*timer_interval/1000;
  printFreq(Code);
  clearLine(1);clearLine(2);
  if(IsToneOn)lcd.print(F("TX "));else lcd.print(F("tx "));
  if(IsToneOn)myOLED.print(F("TX "),OL_DX,MY_OLED_STRING_HIGH+OL_DY);else myOLED.print(F("tx "),OL_DX,MY_OLED_STRING_HIGH+OL_DY); 
  if(percent<10){lcd.setCursor(4,1);oled_x=4;}
  else {lcd.setCursor(3,1); oled_x=3;}
  lcd.print(percent);lcd.print(F("% "));
  myOLED.printNumI(percent,oled_x*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.print(F("% "),(oled_x+2)*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  lcd.print(time_to_end);
  myOLED.printNumI(time_to_end,(oled_x+4)*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  lcd.print(F("s"));
  if(time_to_end<10) oled_x=oled_x+5;
  else if(time_to_end<100)oled_x=oled_x+6;
  else if(time_to_end<1000)oled_x=oled_x+7;
  else oled_x=oled_x+8;
  myOLED.print(F("s"),oled_x*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  lcd.setCursor(13,1);
  if(BEACON) {lcd.print(F("BCN"));
              myOLED.print(F("BCN"),13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
             }
 // if(millis()-last_oled_update>OLED_UPDATE_INTERVAL){myOLED.update();last_oled_update=millis();}
 my_oled_update();
}

void printMode(int mod){
  clearLine(2);
  lcd.setCursor(12,0);
  //if(mode<=OPERATION_MODES) EEPROM.put(EEPROM_MODE_ADR,mode);
  if(mod<=OPERATION_MODES) updwdc(mod,BTN_NONE);
  //if(mod>=MODE_WAIT || mod==MODE_TIME || mod==MODE_QTH) enable_gps(); else disable_gps();
  if(mod==MODE_TX || mod==MODE_EXTERNAL_GPS) disable_gps(); else enable_gps();
  switch(mod){
    case MODE_QRSS: printFreq(CodeQRSS); setF(CodeQRSS, false); lcd.setCursor(12,0);lcd.print(F("QRSS"));
                    myOLED.print(F("QRSS"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msgq);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_qrss);lcd.print('s');
                    myOLED.print(msgq,OL_DX,MY_OLED_STRING_HIGH+OL_DY);myOLED.print(F(" "),12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print("s",15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_qrss,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_DFCW: printFreq(CodeDFCW); setF(CodeDFCW,false);lcd.setCursor(12,0);lcd.print(F("DFCW"));
                    myOLED.print(F("DFCW"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msgd);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_dfcw);lcd.print('s');
                    myOLED.print(msgd,OL_DX,MY_OLED_STRING_HIGH+OL_DY);myOLED.print(F(" "),12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print("s",15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_dfcw,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_HELL: printFreq(CodeHELL); setF(CodeHELL,false);lcd.setCursor(12,0);lcd.print(F("HELL"));
                    myOLED.print(F("HELL"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    clearLine(1); lcd.print(msghl);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_hell);lcd.print('s');
                    myOLED.print(msghl,OL_DX,MY_OLED_STRING_HIGH+OL_DY);myOLED.print(F(" "),12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print("s",15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_hell,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_JT9:  printFreq(CodeJT9); setF(CodeJT9,false);lcd.setCursor(12,0);lcd.print(F("JT-9"));
                    myOLED.print(F("JT-9"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    clearLine(1); lcd.print(msgjt);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_jt9);lcd.print('m');
                    myOLED.print(msgjt,OL_DX,MY_OLED_STRING_HIGH+OL_DY);myOLED.print(F(" "),12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_jt9,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print("m",15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_FW:   printFreq(CodeFST4); setF(CodeFST4,false);lcd.setCursor(12,0);
                    switch(dot_fst4){
                     case 1800:  lcd.print(F("FW30"));
                                 myOLED.print(F("FW30"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                 break;
                     case 900:  lcd.print(F("FW15"));
                                 myOLED.print(F("FW15"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                 break;
                     case 300:  lcd.print(F("FW-5"));
                                 myOLED.print(F("FW-5"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                 break;
                     default:   lcd.print(F("FW-2"));
                                 myOLED.print(F("FW-2"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                 break;
                    } 
                    clearLine(1); clearLine(2);
                    //printFst4msg();
                    lcd.print(call);
                    lcd.setCursor(7,1); lcd.print(qth);
                    lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
                    myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(qth,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_FST4: printFreq(CodeFST4); setF(CodeFST4,false);lcd.setCursor(12,0);lcd.print(F("FST4"));
                    myOLED.print(F("FST4"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    clearLine(1); 
                    printFst4msg();
                    lcd.setCursor(11,1);lcd.print(F("     "));lcd.setCursor(12,1);lcd.print(dot_fst4);
                    myOLED.print(F("     "),11*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_fst4,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_WSPR: printFreq(CodeWSPR); setF(CodeWSPR,false);lcd.setCursor(12,0);lcd.print(F("WSPR"));
                    myOLED.print(F("WSPR"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    clearLine(1); lcd.print(call);
                                  lcd.setCursor(7,1); lcd.print(qth);
                                  lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
                    myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(qth,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_WSPR15: printFreq(CodeWSPR15); setF(CodeWSPR15,false); lcd.setCursor(12,0);lcd.print(F("WR15")); 
                    myOLED.print(F("WR15"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(call);
                                  lcd.setCursor(7,1); lcd.print(qth);
                                  lcd.setCursor(12,1); lcd.print(pwr); lcd.print(F("dB"));
                    myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(qth,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_OP32: printFreq(CodeOP32); setF(CodeOP32,false);lcd.setCursor(12,0); lcd.print(F("OP32"));
                    myOLED.print(F("OP32"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(call);
                    myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_CTRL:  printFreq(CodeCW);setF(CodeCW,false); lcd.setCursor(12,0); lcd.print(F("  CW"));
                    myOLED.print(F("  CW"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    printTonePtt();
                    break;
    case MODE_MSG_QRSS:  clearLine(0); lcd.print(F("MSG QRSS:")); 
                    lcd.print(m);
                    myOLED.print(F("MSG QRSS:"),OL_DX,OL_DY);
                    myOLED.print(m,9*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msgq);
                    myOLED.print(msgq,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_MSG_DFCW:  clearLine(0); lcd.print(F("MSG DFCW:")); 
                    lcd.print(m);
                    myOLED.print(F("MSG DFCW:"),OL_DX,OL_DY);
                    myOLED.print(m,9*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msgd);
                    myOLED.print(msgd,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_MSG_HELL:  clearLine(0); lcd.print(F("MSG HELL:")); 
                    lcd.print(m);
                    myOLED.print(F("MSG HELL:"),OL_DX,OL_DY);
                    myOLED.print(m,9*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msghl);
                    myOLED.print(msghl,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_MSG_JT9:  clearLine(0); lcd.print(F("MSG JT-9:")); 
                    lcd.print(m);
                    myOLED.print(F("MSG JT-9:"),OL_DX,OL_DY);
                    myOLED.print(m,9*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(msgjt);
                    myOLED.print(msgjt,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_MSG_FST4:
                    clearLine(0); lcd.print(F("FST4 TONES:"));
                    myOLED.print(F("FST4 TONES:"),OL_DX,OL_DY);
                    printFst4Menu(fst);
                    clearLine(1); printFst4msg();  
                    break;
    case MODE_CALL: clearLine(0); lcd.print(F("CALL:")); 
                    lcd.print(cc);
                    myOLED.print(F("CALL:"),OL_DX,OL_DY);
                    myOLED.print(cc,5*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    clearLine(1); lcd.print(call);
                    myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_QTH:  clearLine(0); clearLine(1); lcd.setCursor(7,0); lcd.print(F("LOC:")); 
                    lcd.print(qq); 
                    myOLED.print(F("LOC:"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    myOLED.print(qq,11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    lcd.setCursor(7,1); lcd.print(qth);
                    myOLED.print(qth,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_PWR:  clearLine(0); clearLine(1); lcd.setCursor(12,0);lcd.print(F("PWR:"));
                    myOLED.print(F("PWR:"),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    lcd.setCursor(12,1); lcd.print(pwr); lcd.print(F("dB"));
                    myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_DOT_QRSS:  clearLine(0); clearLine(1); lcd.setCursor(7,0);lcd.print(F("QRSS DOT:"));
                    myOLED.print(F("QRSS DOT:"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
                    myOLED.print(F("s"),15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_qrss,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_DOT_DFCW:  clearLine(0); clearLine(1); lcd.setCursor(7,0);lcd.print(F("DFCW DOT:"));
                    myOLED.print(F("DFCW DOT:"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    lcd.setCursor(12,1); lcd.print(dot_dfcw); lcd.print(F("s"));
                    myOLED.print(F("s"),15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_dfcw,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_DOT_HELL:  clearLine(0); clearLine(1); lcd.setCursor(7,0);lcd.print(F("HELL DOT:"));
                    myOLED.print(F("HELL DOT:"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    lcd.setCursor(12,1); lcd.print(dot_hell); lcd.print(F("s"));
                    myOLED.print(F("s"),15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_hell,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_DOT_JT9:  clearLine(0); clearLine(1); lcd.setCursor(0,0); lcd.print(F("JT9 SPEED:")); 
                    myOLED.print(F("JT9 SPEED:"),OL_DX,OL_DY);
                    dotjt9updw();
                    break;
    case MODE_DOT_FST4:  clearLine(0); clearLine(1); lcd.setCursor(0,0); lcd.print(F("FST4 SPEED:")); 
                    myOLED.print(F("FST4 SPEED:"),OL_DX,OL_DY);
                    dotfst4updw();
                    break;
    case MODE_DOT_PAUSE:  clearLine(0); clearLine(1); lcd.setCursor(4,0);lcd.print(F("PAUSE:"));
                    myOLED.print(F("PAUSE:"),4*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    lcd.setCursor(11,1); lcd.print(dot_pause);lcd.print(F("min"));
                    myOLED.print(F("min"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.printNumI(dot_pause,11*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_T_BEACON_START: case MODE_T_BEACON_STOP:
                    clearLine(0); clearLine(1);
                    char s[17]; 
                    sprintf(s,"%02d:%02d ",t_bcn_start_hh,t_bcn_start_mm);
                    lcd.setCursor(0,0); lcd.print(F("BCN START ")); lcd.print(s);
                    myOLED.print(F("BCN START "),0*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    myOLED.print(s,10*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
                    sprintf(s,"%02d:%02d ",t_bcn_stop_hh,t_bcn_stop_mm);
                    lcd.setCursor(0,1);lcd.print(F("BCN STOP  ")); lcd.print(s);
                    myOLED.print(F("BCN STOP  "),0*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY); 
                    myOLED.print(s,10*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    if(mod==MODE_T_BEACON_START){
                      lcd.setCursor(15,0);lcd.print(F("<"));
                      myOLED.print(F("<"),15*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                    }
                    else{
                      lcd.setCursor(15,1);lcd.print(F("<"));
                      myOLED.print(F("<"),15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    }
                    break;
    case MODE_GEN:  clearLine(0);lcd.print(F("CRYSTAL ="));
                    myOLED.print(F("CRYSTAL ="),OL_DX,OL_DY);
                    clearLine(1);lcd.print(Crystal); lcd.print(F(" Hz"));
                    myOLED.printNumI(Crystal,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    myOLED.print(F(" Hz"),13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_CLR_EEPROM:
                    clearLine(0); lcd.print(F("ERASE SETTINGS ?"));
                    myOLED.print(F("ERASE SETTINGS ?"),OL_DX,OL_DY);
                    clearLine(1); lcd.setCursor(7,1); lcd.print(F("NO"));
                    myOLED.print(F("NO"),7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                    break;
    case MODE_TX:   clearLine(0);clearLine(1);
                    paintTX();
                    break;
    case MODE_TIME:  clearLine(0); lcd.print(F("SET TIME [hh:mm]"));
                    myOLED.print(F("SET TIME [hh:mm]"),OL_DX,OL_DY);
                    if (rtc.isrunning()){
                      DateTime now = rtc.now();
                      t_set_hour=now.hour();
                      t_set_min=now.minute();
                      t_set_sec=now.second();
                      setmytime(); //устанавливаем, если время правильное
                    }
                    if(timeIsSet ){ t_set_hour=t_hour; t_set_min=t_min; t_set_sec=t_sec;}
                    clearLine(1); printTime(t_set_hour,t_set_min,t_set_sec);
                    break;
    case MODE_WAIT: case MODE_T60: case MODE_T30: case MODE_PAUSE: case MODE_TSTART:
                    clearLine(0); lcd.print(F("WAITING to "));
                    myOLED.print(F("WAITING to "),OL_DX,OL_DY);
                    if (rtc.isrunning()){
                      DateTime now = rtc.now();
                      t_set_min=now.minute();
                      t_set_sec=now.second();
                      setmytime(); //устанавливаем, если время правильное
                    }
                    if(mod==MODE_T60){lcd.print(F("00:00"));myOLED.print(F("00:00"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                    if(mod==MODE_T30){lcd.print(F("00|30"));myOLED.print(F("00|00"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                    if(mod==MODE_WAIT){lcd.print(F("START"));myOLED.print(F("START"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                    if(mod==MODE_TSTART){char st[20]; sprintf(st,"%02d:%02d:00",t_bcn_start_hh,t_bcn_start_mm);
                                         lcd.setCursor(8,0);lcd.print(st);myOLED.print(st,8*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                         sprintf(st,"BCN to %02d:%02d:00  ",t_bcn_stop_hh,t_bcn_stop_mm);
                                         myOLED.print(st,0*MY_OLED_CHAR_WIDTH+OL_DX,2*MY_OLED_STRING_HIGH+OL_DY);
                    }
                    //if(mod==MODE_PAUSE){lcd.print(F("PAUSE"));myOLED.print(F("PAUSE"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                    if(mod==MODE_PAUSE){long pt=0;
                                        if(t_pause_millis>millis())pt=(t_pause_millis-millis())/1000;
                                        lcd.print(pt);lcd.print(F(" s"));
                                        myOLED.printNumI(pt,11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
                                        myOLED.print(F(" s"),13*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                    clearLine(1); printTime(t_hour,t_min, t_sec);
                    break;
    case MODE_BEACON:  clearLine(0); lcd.print(F("BEACON:"));
                    myOLED.print(F("BEACON:"),OL_DX,OL_DY);
                    printBcnMenu(bcn);
                    clearLine(1); printBeaconList(); 
                    break;
    case MODE_SUBMENU: clearLine(0); 
                    if(submenu){ lcd.print(F("Exit SETUP..."));myOLED.print(F("Exit SETUP..."),CENTER,OL_DY);} 
                    else {lcd.print(F("SETUP..."));myOLED.print(F("SETUP..."),CENTER,OL_DY);}
                    clearLine(1);
                    break;
    case MODE_LF2:  lf2updw();
                    clearLine(1);
                    break;
    case MODE_BORDERS_LF: borderslfupdw();
                    clearLine(1);
                    break;
    case MODE_TIMER_CORRECTION:
                    clearLine(0); lcd.print(F("dT[s] per 10min:"));
                    myOLED.print(F("dT[s] per 10min:"),OL_DX,OL_DY);
                    timcorupdw();
                    break;
    case MODE_GPS:  enable_gps();
                    printGPS();
                    break;
    case MODE_EXTERNAL_GPS:
                    Serial.begin(9600);
                    clearLine(0); lcd.print(F("USB link to GPS"));
                    myOLED.print(F("USB link to GPS"),CENTER,OL_DY);
                    clearLine(1); lcd.print(F("USB:     GPS:  "));
                    //myOLED.print(F("USB:     GPS:  "),LEFT,MY_OLED_STRING_HIGH);
                    break;
    case MODE_LOAD_FST4:
                    Serial.begin(9600);
                    clearLine(0); lcd.print(F("LOAD FST4 TONES:"));
                    myOLED.print(F("LOAD FST4 TONES:"),CENTER,OL_DY);
                    clearLine(1); 
                    break;
    case MODE_EXTERNAL_SYNT:
                    if(GMDDS) Serial.begin(115200);else Serial.begin(9600);
                    clearLine(0); lcd.print(F("USB link to SYNT"));
                    myOLED.print(F("USB link to SYNT"),CENTER,OL_DY);
                    clearLine(1); lcd.print(F("USB:    SYNT:  "));
                    //myOLED.print(F("USB:    SYNT:  "),LEFT,MY_OLED_STRING_HIGH);
                    break;
    default:   ;//lcd.print(F("????"));
  } 
  myOLED.update();
}
/*
void printCode(unsigned long cd){
  lcd.setCursor(0,0);
  lcd.print(cd,HEX);
}
*/
void printVbat(void){
  static double Uold;
  double U=analogRead(A3);
  static unsigned long timer_correction_old;
  U=U*50.0/1024;
  if(Uold==U && timer_correction_old==timer_correction) return;
  Uold=U;
  timer_correction_old=timer_correction;
  //lcd.setCursor(12,1);
  //lcd.print(U);
  //lcd.setCursor(15,1);
  //lcd.print('V');
  //myOLED.drawLine(0,OL_DY+2*MY_OLED_STRING_HIGH,127,OL_DY+2*MY_OLED_STRING_HIGH);
  clearLine(2);
  if(U>3 && U<20)
  {myOLED.printNumF(U,1,OL_DX,OL_DY+2*MY_OLED_STRING_HIGH);
   myOLED.print(F("v"),OL_DX+4*MY_OLED_CHAR_WIDTH,OL_DY+2*MY_OLED_STRING_HIGH);
  }
  myOLED.printNumF(0.6*timer_correction,3,OL_DX + 7*MY_OLED_CHAR_WIDTH,2*MY_OLED_STRING_HIGH+OL_DY);
  
 // if(foundRTC)myOLED.print(F("RTC"),OL_DX+9*MY_OLED_CHAR_WIDTH,OL_DY+2*MY_OLED_STRING_HIGH);
  if(gps_detected)myOLED.print(F("GPS"),OL_DX+14*MY_OLED_CHAR_WIDTH,OL_DY+2*MY_OLED_STRING_HIGH);
  //myOLED.update();
}
void printGPS(void)
{
  clearLine(0);
  char s[18]; strncpy(s,gps_s,16);
  lcd.print(s);
  myOLED.print(s,OL_DX,OL_DY);
  printTime(t_hour,t_min,t_sec);
  lcd.print(' ');
  lcd.print(qth);
  myOLED.print(" ",12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.print(qth,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
   clearLine(2); 
   myOLED.print(F("calib="),OL_DX+1*MY_OLED_CHAR_WIDTH,OL_DY+2*MY_OLED_STRING_HIGH);
   myOLED.printNumF(0.6*timer_correction,3,OL_DX + 7*MY_OLED_CHAR_WIDTH,2*MY_OLED_STRING_HIGH+OL_DY);
  //myOLED.update();
}
void printBeaconList(void)
{
 clearLine(1);clearLine(2);
 lcd.print(bcn_list);
 myOLED.print(bcn_list,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.update();
}
void printFst4msg(void)
{
 clearLine(1);clearLine(2);
 if(msgfst4_pos<11){
  lcd.print(msgfst4);
  myOLED.print(msgfst4,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 }
 else{
  lcd.print(&msgfst4[msgfst4_pos-11]);
  myOLED.print(&msgfst4[msgfst4_pos-11],OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 }
 lcd.setCursor(13,1); lcd.print(msgfst4_pos);
 myOLED.printNumI(msgfst4_pos,OL_DX+13*MY_OLED_CHAR_WIDTH,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.update();
 //my_oled_update();
}

void my_oled_update(void)
{
 if(millis()-last_oled_update>MY_OLED_UPDATE_INTERVAL){myOLED.update();last_oled_update=millis();}
}

void printTime(byte hh, byte mm, byte ss)
{
 byte h1,h2,m1,m2,s1,s2;
 h1=hh%10; h2=(hh-h1)/10;
 m1=mm%10; m2=(mm-m1)/10;
 s1=ss%10; s2=(ss-s1)/10;
 lcd.setCursor(0,1);
 if(gps_ready) {lcd.print(F("GPS="));myOLED.print(F("GPS="),OL_DX,MY_OLED_STRING_HIGH+OL_DY);gps_ready=false;}
 else {if(gps_detected) {lcd.print(F("GPS "));myOLED.print(F("GPS "),OL_DX,MY_OLED_STRING_HIGH+OL_DY);gps_detected=false;}
       else  {lcd.print(F("    "));myOLED.print(F("    "),OL_DX,MY_OLED_STRING_HIGH+OL_DY);} 
 }
 lcd.setCursor(4,1); 
 lcd.print(h2,DEC);
 lcd.print(h1,DEC);
 lcd.print(':');
 lcd.print(m2,DEC);
 lcd.print(m1,DEC);
 lcd.print(':');
 lcd.print(s2,DEC);
 lcd.print(s1,DEC);
 myOLED.printNumI(h2,4*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.printNumI(h1,5*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.print(":",6*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.printNumI(m2,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.printNumI(m1,8*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.print(":",9*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.printNumI(s2,10*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 myOLED.printNumI(s1,11*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);

 if(rtc.isrunning()){
  lcd.print(F(" RTC"));
  myOLED.print(F(" RTC"),12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
 }
 if(!timeIsSet){myOLED.print(F("TIME is NOT set"),0*MY_OLED_CHAR_WIDTH+OL_DX,2*MY_OLED_STRING_HIGH+OL_DY);}
 else { clearLine(2); 
        if(mode==MODE_TSTART){char st[20]; sprintf(st,"BCN to %02d:%02d:00  ",t_bcn_stop_hh,t_bcn_stop_mm);
          myOLED.print(st,0*MY_OLED_CHAR_WIDTH+OL_DX,2*MY_OLED_STRING_HIGH+OL_DY);
        }
 }
 myOLED.update();
}

bool readF(void)
{
   unsigned char s[8];
   memset(s,0,sizeof(s));
   int N;
   if(GMDDS)
   {N=Serial3.readBytes(s,4);
    //if(N==0) return false;
    if(N!=4) return false;
             
     unsigned long x1,x2,x3,x4;
     unsigned long y=0;
     x1=x2=x3=x4=0;
     y=s[0]; x1=(y<<24)&0xff000000; 
     y=s[1]; x2=(y<<16)&0x00ff0000; 
     y=s[2]; x3=(y<<8)&0x0000ff00; 
     x4=(s[3])&0x000000ff;
     if(s[0]&0x80) //это команда и фаза
     {
     }
     else
     {
      Code=x1 + x2  + x3 +  x4;
      if(LF2) Code=Code>>1;
     }
    return true;
   }

   N=Serial3.readBytes(s,1);
   if(N==0) return false;
   if(s[0]=='F')
   { 
M1:    
     unsigned long x1,x2,x3; 
     N=Serial3.readBytes(&s[1],5); //дочитать байты
     if(N!=5) return false;
     if(s[1]>0)
     { DDS_VER=(s[1]&0xe0)>>5;
       if(s[1]&0x01) IsPttOn=1;//PTT отобразить
       else IsPttOn=0;
       if(s[1]&0x10) set32bit();//32 bit mode
       else set24bit();
     }
     else
     {DDS_VER=0;
      set24bit();
     }
     unsigned long y=0;
     x1=x2=x3=0;
     if(BIT32){
       y=s[2]; x1=(y<<24)&0xff000000; 
       y=s[3]; x2=(y<<16)&0x00ff0000; 
       y=s[4]; x3=(y<<8)&0x0000ff00;
     }
     else {
      y=s[2]; x1=(y<<16)&0xff0000; 
      y=s[3]; x2=(y<<8)&0x00ff00; 
      x3=(s[4])&0x0000ff;
     }
     Code=x1 + x2  + x3 +  b0_32;
     if(LF2) Code=Code>>1;
   }
   else
   {
    if(N==0) return false;
    int rr=Serial3.available();
    for(int i=0; i<rr; i++)
    {
      char cccc=0; Serial3.readBytes(&cccc,1);
      if(cccc=='F') goto M1;
    }
    Serial3.begin(9600);
    Serial3.setTimeout(300);
    return false;
   }
   return true;
}
bool setF(unsigned long code, bool invPh)
{ Code=code;
  if(LF2) code=code<<1;
   unsigned long CC=code;
    unsigned char c[8];
  memset(c,0,sizeof(c));
  bool result=false;
/*
  if(DDSAD)
  {
    dds_set(code); //Code=code;
    return true;
  }
 */
 
  if(GMDDS)
  { if(invPh)
    {
     c[0]=0xc0 | (GMPTT<<4) | (GMTONE<<5);
     c[1]=(0xff00 & GMPH)>>8;
     c[2]=(0x00ff & GMPH);
    }
    else
    {
     CC=code; c[0]=0x7f & (CC>>24);
     CC=code; c[1]=0xff & (CC>>16);
     CC=code; c[2]=0xff & (CC>>8);
     CC=code; c[3]=0xff & (CC);
    }
     result=false;
     Serial3.write(c,4);
     result=readF();
   return result;
  }
   c[0]='s';
   if(BIT32)
   { CC=code; c[1]=0xff & (CC);
     b0_32=c[1];
     CC=code; c[2]=0xff & (CC>>24);
     CC=code; c[3]=0xff & (CC>>16);
     CC=code; c[4]=0xff & (CC>>8);
   }
   else
   { c[1]=0;
     b0_32=c[1];
     CC=code; c[2]=0xff & (CC>>16);
     CC=code; c[3]=0xff & (CC>>8);
     CC=code; c[4]=0xff & (CC>>0);
   }
   c[5]=0;
   result=true;
   Serial3.write(c,5);
   result=readF();
   if(!result) printError("NO DDS");
   return result;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// JT-9 ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
int jt9_coding(char message[])
{
 unsigned long poly1=-221228207;//0xF2D05351;
 unsigned long poly2=-463389625;//0xE4613C47;
 const static byte partab[] PROGMEM= { 0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0,
                0, 1, 1, 0, 1, 0, 0, 1,
                0, 1, 1, 0, 1, 0, 0, 1,
                1, 0, 0, 1, 0, 1, 1, 0};

 const static byte isync[] PROGMEM ={1,1,0,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,
             0,0,1,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,
             0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,0,0,0,0,1,
             0,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,
             0,0,1,0,1};

 long  i4Msg6BitWords[13];
 long  i1Msg8BitBytes[13];
 int  i1EncodedBits[207];   //long
 int  i1ScrambledBits[207]; //long
 long i4DataSymbols[69];
 long i4GrayCodedSymbols[69];
 long i4tone[85];
 bool text;
 const char abcj[44]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ +-./?'\x0";

 for(int i=0; i<13; i++)if(message[i]==0)message[i]=' ';
 message[13]=0;
 //packmsg()
 for(int i=0; i<13; i++) i4Msg6BitWords[i]=0;
 {
 long nc1=0;
 long nc2=0;
 long nc3=0;
 int j=0;
 for(int i=0;i<5;i++)
 { for(j=0;j<44;j++) if(message[i]==abcj[j])break;
     if(j==44)j=36;
     nc1=42*nc1+j;
 }
 for(int i=5;i<10;i++)
 { for(j=0;j<44;j++) if(message[i]==abcj[j])break;
     if(j==44)j=36;
     nc2=42*nc2+j;
 }
 for(int i=10;i<13;i++)
 { for(j=0;j<44;j++) if(message[i]==abcj[j])break;
     if(j==44)j=36;
     nc3=42*nc3+j;
 }
 nc1=nc1+nc1;
 if((nc3&32768L)!=0) nc1=nc1+1;
 nc2=nc2+nc2;
 if((nc3&65536L)!=0) nc2=nc2+1;
 nc3=nc3&32767L;

 long ng=nc3;
 ng=ng+32768L;

 i4Msg6BitWords[0]=(nc1>>22)&63;
 i4Msg6BitWords[1]=(nc1>>16)&63;
 i4Msg6BitWords[2]=(nc1>>10)&63;
 i4Msg6BitWords[3]=(nc1>>4)&63;
 i4Msg6BitWords[4]=(4*(nc1&15)) + ((nc2>>26)&3);
 i4Msg6BitWords[5]=(nc2>>20)&63;
 i4Msg6BitWords[6]=(nc2>>14)&63;
 i4Msg6BitWords[7]=(nc2>>8)&63;
 i4Msg6BitWords[8]=(nc2>>2)&63;
 i4Msg6BitWords[9]=(16*(nc2&3)) + ((ng>>12)&15);
 i4Msg6BitWords[10]=(ng>>6)&63;
 i4Msg6BitWords[11]=(ng)&63;
 }
 // pack 6 bit simbols to 8 bit symbols + zero tail
 {
 int k=0;
 int m=0;
 int i4=0;
 for(int i=0; i<13; i++) i1Msg8BitBytes[i]=0;
 for(int i=0; i<12; i++)
 { int n=i4Msg6BitWords[i];
   for(int j=1;j<=6;j++)
   { k=k+1;
     i4=i4+i4+ (1&(n>>(6-j)));
     i4=i4&255;
     if(k==8)
     {if(i4>127)i4=i4-256;
      i1Msg8BitBytes[m]=i4;
      m=m+1;
      k=0;
     }
   }
 }
 }
// Conv coding
 {
   long nstate=0;
   int k=0;
   long i1,i4;
   for (int j=0; j<13; j++)
   {
      for(int i=7;i>=0;i--)
      { i1=i1Msg8BitBytes[j];
        i4=i1;
        if(i4<0)i4=i4+256;
        nstate= (nstate<<1) | ((i4>>i)&1);
        long n= nstate & poly1;
        n= n ^ (n>>16);
        long nn= (n ^ (n>>8)) & 255;
        i1EncodedBits[k]=pgm_read_byte(&partab[nn]);
        k=k+1;
        n= nstate & poly2;
        n= n ^ (n>>16);
        nn= (n ^ (n>>8)) & 255;
        i1EncodedBits[k]=pgm_read_byte(&partab[nn]);
        k=k+1;
        if(k>=206)break;
      }
      if(k>=206)break;
   }
 }

 //interleave
 {
  int j0[206];
  bool first=true;
  if(first)
  { int k=0;
    for(int i=0; i<256; i++)
    { int m=i;
      int n=m & 1;
      n=2*n + ((m/2)&1);
      n=2*n + ((m/4)&1);
      n=2*n + ((m/8)&1);
      n=2*n + ((m/16)&1);
      n=2*n + ((m/32)&1);
      n=2*n + ((m/64)&1);
      n=2*n + ((m/128)&1);
      n=n&0xff;
      if(n<206){j0[k]=n;k=k+1;}
    }
  }
  for(int i=0; i<206;i++) i1ScrambledBits[j0[i]]= i1EncodedBits[i];
 }

 //pack 3bits into word
 {
   int k=0;
   for(int i=0; i<69; i++)
   {  int n=0;
      for(int j=0; j<3; j++)
      {
       int m=i1ScrambledBits[k];
       n=n<<1 | m;
       k=k+1;
       if(k>=206)break;
      }
      i4DataSymbols[i]=n;
   }
 }

 //Gray code
 for(int i=0; i<69; i++)
 {
   int n=i4DataSymbols[i];
   i4GrayCodedSymbols[i]=(n ^ (n >> 1));
 }
 // Insert sync symbols at ntone=0 and add 1 to the data-tone numbers.
 {int j=0;
  for(int i=0;i<85;i++)
  {  if(pgm_read_byte(&isync[i])==1) i4tone[i]=0;
     else {i4tone[i]=i4GrayCodedSymbols[j]+1; j=j+1;}
  }
 }
 //к строковому виду
 memset(tx,0,sizeof(tx));
 for(int i=0;i<85;i++) tx[i]=i4tone[i]+1+0x30;
 return 85;
}
//------------------------------------------------------------------------------
////////////////////// END JT-9 /////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const int order = 16;
const unsigned long polynom = 0x8005UL;
const unsigned long crcxor = 0x0UL;
//const int refin = 1;
//const int refout = 1;

unsigned long crcmask;
//unsigned long crcinit_direct;
unsigned long crchighbit;

unsigned long reflect (unsigned long crc, int bitnum) {
  // reflects the lower 'bitnum' bits of 'crc'
  unsigned long i, j=1, crcout=0;
  for (i=(unsigned long)1<<(bitnum-1); i; i>>=1) {
    if (crc & i) crcout|=j;
    j<<= 1;
  }
  return (crcout);
}

unsigned long crcbitbybitfast(char* p, unsigned long len) {

  // fast bit by bit algorithm without augmented zero bytes.
  // does not use lookup table, suited for polynom orders between 1...32.

  unsigned long i, j, c, b;
  unsigned long crc = 0UL;//crcinit_direct;

  for (i=0; i<len; i++) {
    c = (unsigned long)*p++;
    /*if (refin)*/ c = reflect(c, 8);

    for (j=0x80; j; j>>=1) {
      b = crc & crchighbit;
      crc<<= 1;
      if (c & j) b^= crchighbit;
      if (b) crc^= polynom;
    }
  }
  /*if (refout)*/ crc=reflect(crc, order);
  crc^= crcxor;
  crc&= crcmask;
  return(crc);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int opera_coding(void)
{memset(tx,0,sizeof(tx)); tx_pos=0;
 char c[7];
 memset(c,0,sizeof(c));
 strcpy(c,call);
  // подготовить позывной
 if(strlen(c)<6){
  if(!isdigit(c[2])) {memmove(&c[1],&c[0],5*sizeof(char));c[0]=' ';c[6]=0;}
 }
 byte len=strlen(c);
 switch(len)
 { case 3: c[3]=' ';c[4]=' ';c[5]=' ';c[6]=0;break;
   case 4: c[4]=' ';c[5]=' ';c[6]=0; break;
   case 5: c[5]=' ';c[6]=0; break;
 }
 if(!isdigit(c[2])){printError("BAD CALL"); return 0;}

 
 // encoding mapping
 // ( ( ( ( (1) * 36 + 0) * 10 + 1) * 27 + 1) * 27 + 1) * 27 + 0
 if(c[0]==' ') c[0]=0;
 if(c[0]>47 && c[0]<58) c[0]=c[0]-21; // 0...9
 if(c[0]>64 && c[0]<91) c[0]=c[0]-64; // A...Z

 if(c[1]>47 && c[1]<58) c[1]=c[1]-22; // 0...9
 if(c[1]>64 && c[1]<91) c[1]=c[1]-65; // A...Z

 if(c[2]>47 && c[2]<58) c[2]=c[2]-48; // 0...9

 if(c[3]==' ') c[3]=0;
 if(c[3]>64 && c[3]<91) c[3]=c[3]-64; // A...Z

 if(c[4]==' ') c[4]=0;
 if(c[4]>64 && c[4]<91) c[4]=c[4]-64; // A...Z

 if(c[5]==' ') c[5]=0;
 if(c[5]>64 && c[5]<91) c[5]=c[5]-64; // A...Z

 uint64_t map=0ULL;
 //map=(((((c[0])*36 + c[1]) * 10 + c[2]) * 27 + c[3]) * 27 + c[4]) * 27 + c[5];
 map=c[1];
 map=map + c[0]*36;
 map=map*10;
 map=map + c[2];
 map=map*27;
 map=map + c[3];
 map=map*27;
 map=map+c[4];
 map=map*27;
 map=map+c[5];
   
 unsigned long crc;
 char msg51[51];
 char arr[47];
 for(int i=0; i<28; i++) {if(1 & (map>>i)) arr[27-i]='1'; else arr[27-i]='0';}
 crcmask = ((((unsigned long)1<<(order-1))-1)<<1)|1;
 crchighbit = (unsigned long)1<<(order-1);
 crc=crcbitbybitfast(arr, 28);
 unsigned int crc_low,crc_high;
 crc_low=crc&0xff;
 crc_high=(crc>>8)&0xff;
 if(crc_low==0)crc_low=0x1b;
 if(crc_high==0)crc_high=0x2b;
 crc=(crc_low<<8)^crc_high;
 for(int i=0; i<16; i++) {if(1 & (crc>>i)) arr[27+16-i]='1'; else arr[27+16-i]='0';}
 crc=crcbitbybitfast(arr, 28+16);
 crc_low=crc&0xff;
 crc_high=(crc>>8)&0xff;
 if(crc_low==0)crc_low=0x1b;
 if(crc_high==0)crc_high=0x2b;
 crc=(crc_low<<8)^crc_high;
 crc=crc&0x07;
 for(int i=0; i<3; i++) {if(1 & (crc>>i)) arr[27+16+3-i]='1'; else arr[27+16+3-i]='0';}
 char nvect[]="111000010101011111100110110100000001100100010101011";
 for(int i=0; i<28+16+3; i++) msg51[i+4]=arr[i];
 for(int i=0; i<4; i++) msg51[i]='0';
 for(int i=0; i<51; i++)
 { if(msg51[i]==nvect[i]) msg51[i]='0';
   else msg51[i]='1';
 }

 char wh_coded[17][8];
 for(int i=0; i<51; i=i+3)
 {
  if(strncmp(&msg51[i],"000",3)==0)strncpy(&wh_coded[i/3][0],"0000000",7);
  if(strncmp(&msg51[i],"001",3)==0)strncpy(&wh_coded[i/3][0],"1010101",7);
  if(strncmp(&msg51[i],"010",3)==0)strncpy(&wh_coded[i/3][0],"0110011",7);
  if(strncmp(&msg51[i],"011",3)==0)strncpy(&wh_coded[i/3][0],"1100110",7);
  if(strncmp(&msg51[i],"100",3)==0)strncpy(&wh_coded[i/3][0],"0001111",7);
  if(strncmp(&msg51[i],"101",3)==0)strncpy(&wh_coded[i/3][0],"1011010",7);
  if(strncmp(&msg51[i],"110",3)==0)strncpy(&wh_coded[i/3][0],"0111100",7);
  if(strncmp(&msg51[i],"111",3)==0)strncpy(&wh_coded[i/3][0],"1101001",7);
 }
 char ilv[119]; 
 int k=0;
 for(int i=0; i<7; i++)
 {
  for(int j=0; j<17; j++) {ilv[k]=wh_coded[j][i];k++;}
 }
// tx="11"; 
 tx[0]='1'; tx[1]='1';tx_pos=2;
 for(int i=0; i<119; i++)
 {
  if(ilv[i]=='0'){tx[tx_pos]='1';tx[tx_pos+1]='0';tx_pos=tx_pos+2;}// tx+="10"; 
  else {tx[tx_pos]='0';tx[tx_pos+1]='1';tx_pos=tx_pos+2;}//tx+="01";
 }
  tx[239]=0;
 return 239;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int wspr_coding(void)
{memset(tx,0,sizeof(tx)); tx_pos=0;
 char wsprcall[7];
 unsigned long poly1=0xF2D05351UL;
 unsigned long poly2=0xE4613C47UL;
 const static byte sync[] PROGMEM={1,1,0,0,0,0,0,0,1,0,0,0,1,1,1,0,0,0,1,0,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1,0,0,1,0,1,0,0,
0,0,0,0,1,0,1,1,0,0,1,1,0,1,0,0,0,1,1,0,1,0,0,0,0,1,1,0,1,0,1,0,1,0,1,0,0,1,0,0,1,0,
1,1,0,0,0,1,1,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,0,1,0,0,1,1,1,0,1,1,0,0,1,1,0,1,0,0,0,1,
1,1,0,0,0,0,0,1,0,1,0,0,1,1,0,0,0,0,0,0,0,1,1,0,1,0,1,1,0,0,0,1,1,0,0,0};
memset(wsprcall,0,sizeof(wsprcall));
strcpy(wsprcall,call);
  // подготовить позывной
 if(strlen(wsprcall)<6){
  if(!isdigit(wsprcall[2])) {memmove(&wsprcall[1],&wsprcall[0],5*sizeof(char));wsprcall[0]=' ';wsprcall[6]=0;}
 }
 byte len=strlen(wsprcall);
 switch(len)
 { case 3: wsprcall[3]=' ';wsprcall[4]=' ';wsprcall[5]=' ';wsprcall[6]=0;break;
   case 4: wsprcall[4]=' ';wsprcall[5]=' ';wsprcall[6]=0; break;
   case 5: wsprcall[5]=' ';wsprcall[6]=0; break;
 }
 if(!isdigit(wsprcall[2])){printError("BAD CALL"); return 0;}

 // сформировать сжатый callsign
 //char abc[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ ";
 char c[7];
 memset(c,0,sizeof(c));
 for(int k=0; k<6; k++){for(int i=0; i<37; i++) if(wsprcall[k]==abc[i]){c[k]=i;}}
 uint64_t N=0ULL;//(((((c[0])*36 + c[1]) * 10 + c[2]) * 27 + c[3]-10) * 27 + c[4]-10) * 27 + c[5]-10;
  N=c[1];
  N=N+c[0]*36;
  N=N*10;
  N=N+c[2];
  N=N*27;
  N=N+c[3]-10;
  N=N*27;
  N=N+c[4]-10;
  N=N*27;
  N=N+c[5]-10;

 if(!isalpha(qth[0]) || !isalpha(qth[1]) || !isdigit(qth[2]) || !isdigit(qth[3])){printError("BAD LOC"); return 0;}

 int nlong= 180 - 20*(qth[0]-64-1);
 int n20d= 2*(qth[2]-48+1);
 nlong=nlong-n20d;
 int lat= 10*(qth[1]-64-1) + (qth[3]-48);
 uint64_t M=((nlong+180)/2)*180 +lat;

 // Power Level
 
 M=M*128 + pwr +64;
 N= ((N<<22) | (M&0x3fffffULL)) & 0x3ffffffffffffULL;

 // Conv coding
 unsigned long reg=0UL;
 uint64_t mask=1ULL;
 mask=mask<<49;
 for (int i=0; i<50; i++)
 {  reg=reg<<1;
    int b=0;
    if(N&mask) b=1;
    reg=reg | b;
    unsigned long r=0UL;
    b=0;
    r=reg&poly1;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b){tx[tx_pos]='1';tx_pos++;}//tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;}//tx+="0";
    b=0;
    r=reg&poly2;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b){tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;} //tx+="0";
    mask=mask>>1;
   // N=N<<1;
 }
  for (int i=0; i<31; i++)
 {  reg=reg<<1;
    unsigned long r=0;
    int b=0;
    r=reg&poly1;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b) {tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;} //tx+="0";
    b=0;
    r=reg&poly2;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b) {tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else  {tx[tx_pos]='0';tx_pos++;} //tx+="0";
 }

 //Interliving
 char d[163];
 int p=0;
 for(int i=0; i<256; i++)
 { int j=0;
   // bit reversing
   j= ((i&128)>>7) | ((i&64)>>5) | ((i&32)>>3) | ((i&16)>>1) | ((i&8)<<1) | ((i&4)<<3) | ((i&2)<<5) | ((i&1)<<7);
   if(j<162){d[j]=tx[p]; p++;}
   if(p==162) break;
 }

 //merge with sync vector
 memset(tx,0,sizeof(tx));tx_pos=0;
 byte snc=0;
 for(int i=0; i<162; i++) {snc=pgm_read_byte(&sync[i]); tx[tx_pos]=1+ 48+ snc +2*(d[i]-48); tx_pos++;} 
 return 162;
}

//---------------------------------------------------------------------------
int qrss_coding(char s[])
{
 int len=0;
 char cc[7];
 tx_len=0;  memset(tx,0,sizeof(tx));
 len=strlen(s);
 char d[7];
 for(int i=0; i<len; i++)
 {
  strcpy(cc,char_to_dfcw(s[i]));
  int cc_len=strlen(cc);
  if(s[i]==' ' || s[i]=='_') {strcpy(d,"0000"); //пауза между словами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
  }
  else{
    for(int k=0;k<cc_len;k++) { // символ
     if(cc[k]=='-') strcpy(d,"1110");
     if(cc[k]=='.') strcpy(d,"10");
     if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
     strcat(tx,d);
    }// символ кончился
   strcpy(d,"00");//пауза между символами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
   }
 }
 //сообщение кончилось - добавить паузу после него
 //  strcpy(d,"0000"); //пауза между словами
 //  if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
 //  strcat(tx,d);
   tx_len=strlen(tx);
   return tx_len;
}

int dfcw_coding(char s[])
{
 int len=0;
 char cc[7];
 tx_len=0; memset(tx,0,sizeof(tx));
 len=strlen(s);
 char d[7];
 for(int i=0; i<len; i++)
 {
  strcpy(cc,char_to_dfcw(s[i]));
  int cc_len=strlen(cc);
  if(s[i]==' ' || s[i]=='_') {strcpy(d,"00000"); //пауза между словами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
  }
  else{
    for(int k=0;k<cc_len;k++) { // символ
     if(cc[k]=='-') strcpy(d,"2220");
     if(cc[k]=='.') strcpy(d,"1110");
     if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
     strcat(tx,d);
    }// символ кончился
   strcpy(d,"00");//пауза между символами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
   }
 }
 //сообщение кончилось - добавить паузу после него
 //  strcpy(d,"00000"); //пауза между словами
 //  if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
 //  strcat(tx,d);
   tx_len=strlen(tx);
   return tx_len;
}

int hell_coding(char s[])
{
 int len=0;
 char cc[30];
 tx_len=0; memset(tx,0,sizeof(tx));
 len=strlen(s);
 for(int i=0; i<len; i++)
 {
  strcpy(cc,char_to_hell(s[i]));
  if(strlen(tx)+strlen(cc)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
  strcat(tx,cc);
 }
   tx_len=strlen(tx);
   return tx_len;
}

int fst4_coding(char s[])
{
 int len=0;
 char cc[30];
 tx_len=0; memset(tx,0,sizeof(tx));
 len=strlen(s);
 for(int i=0; i<len; i++)
 {tx[i]=msgfst4[i]+1;
 }
   tx_len=strlen(tx);
   return tx_len;
}

bool enable32(void)
{
  if(GMDDS)return true;
  //bool result=false;
  for(int i=0; i<3;i++)
  {Serial3.write("4",1);
   readF();
   delay(30);
  }
  return BIT32;
}

bool enable24(void)
{ if(GMDDS)return false;
  bool result=false;
  for(int i=0; i<3;i++)
  {Serial3.write("3",1);
   result=readF();
   delay(30);
  }
  return result;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void timer1(){
  if(tx_pos==0){mode=MODE_TX; paintTX(); disable_gps(); pttON(); delay(100); toneON();} //было 1c - пауза перед WSPR. Не помешает и другим модам.
  if(mode!=MODE_TX)return;
  if((tx_pos>0 && tx[tx_pos]!=tx[tx_pos-1]) || tx_pos==0){
  switch(mode_old)
  {case MODE_QRSS: case MODE_OP32:
    if(tx[tx_pos]=='0') toneOFF();else toneON();
    break;
   case MODE_DFCW: 
    if(tx[tx_pos]=='0') toneOFF();
    else{
    Code=CodeDFCW+(tx[tx_pos]-0x30-1)*tx_dc;
    setF(Code,false);
    toneON();
    }
    break;
   case MODE_HELL: 
    if(tx[tx_pos]=='0') toneOFF();
    else{
      Code=CodeHELL+(tx[tx_pos]-0x30-1)*tx_dc;
      setF(Code,false);
      toneON();
    }
    break;
   case MODE_WSPR:
    Code=CodeWSPR+(tx[tx_pos]-0x30-1)*tx_dc;
    /*if(DDS_VER>0 && !BIT32) setFastF(tx[tx_pos]-0x30-1);
    else*/ setF(Code,false);
    break;
   case MODE_WSPR15:
    Code=CodeWSPR15+(tx[tx_pos]-0x30-1)*tx_dc;
    /*if(DDS_VER>0 && !BIT32) setFastF(tx[tx_pos]-0x30-1);
    else*/ setF(Code,false);
    break;
   case MODE_FW:
   case MODE_FST4:
    Code=CodeFST4+(tx[tx_pos]-0x30-1)*tx_dc;
    /*if(DDS_VER>0 && !BIT32) setFastF(tx[tx_pos]-0x30-1);
    else*/ setF(Code,false);
    break;
   case MODE_JT9:
    Code=CodeJT9+(tx[tx_pos]-0x30-1)*tx_dc;
    setF(Code,false);
    break;
   default: ;
  }
  }
  tx_pos++;
  if(tx_pos>tx_len){
    tx_len=0; toneOFF();pttOFF();
    enable_gps();
    compute_time();    
    if(BEACON) {bcn_pos++; start_beacon(bcn_pos);}
    else {mode=mode_old; mode_old=MODE_TX;}
  }
}


//////////////////////////////////////////////////////////////////////
void init_dds(void)
{ bool result=false;
  for (int i=0; i<3; i++)
  { if(i==0)Serial3.write("H");else Serial3.write("?");
    result=readF();
    freq=Code*Step;
    if(result) 
    {
      if(DDS_VER>=3) {set32bit(); enable32();freq=Code*Step;}
      if(i>0)break;
    }
    delay(30);
  }
}
//////////////////////////////////////////////////////////////////////

void setup() {
  // put your setup code here, to run once:
//  setup_AD98xx_pins();
  freq=137500.0;
  //set24bit();
  Serial3.begin(9600);
  Serial3.setTimeout(100);
  Serial.begin(9600);
  Serial.setTimeout(100);
  Serial1.begin(9600);
  Serial1.setTimeout(100);

  lcd.begin(16, 2);
  lcd.print(F("RN3AUS DDS CTRL"));
 
  myOLED.begin();
  myOLED.setFont(SmallFont);
  myOLED.clrScr();
  myOLED.print(F("RN3AUS DDS CTRL"), CENTER, OL_DY);
  myOLED.update();

  #ifdef BOARD_UNO
   lcd.setCursor(2,1);
   lcd.print(F("VERSION 1.2"));
  #endif
  
  #ifdef BOARD_MEGA
   lcd.setCursor(0,1);
   lcd.print(F("VERSION 5.2 MEGA"));
   myOLED.print(F("VERSION 5.2 MEGA"), CENTER, MY_OLED_STRING_HIGH+OL_DY);
   myOLED.update();
  
   Serial2.begin(9600);  //GPS
   Serial2.setTimeout(100);
  #endif

  delay(1000);
  if (rtc.begin()) {
    clearLine(1);
    lcd.setCursor(1,1);
    lcd.print(F("RTC found"));
    myOLED.print(F("RTC found"), CENTER, MY_OLED_STRING_HIGH+OL_DY);
    myOLED.update();
  }
 
  
  EEPROM.get(EEPROM_CALL_ADR,call);
  if(call[0]<0 || call[0]>125) memset(call,0,sizeof(call));
  call_pos=strlen(call);
  EEPROM.get(EEPROM_QTH_ADR,qth);
  if(qth[0]<0 || qth[0]>125) memset(qth,0,sizeof(qth));
  qth_pos=strlen(qth);
  EEPROM.get(EEPROM_PWR_ADR,pwr);
  if(pwr>60 || pwr<0) pwr=0;
  for(int i=0; i<19; i++) if(pwr==pwr_list[i]) n_pwr=i;
  
  EEPROM.get(EEPROM_MSG_QRSS_ADR,msgq);
  if(msgq[0]<0 || msgq[0]>125) memset(msgq,0,sizeof(msgq));
  msgq_pos=strlen(msgq);
  
  EEPROM.get(EEPROM_MSG_DFCW_ADR,msgd);
  if(msgd[0]<0 || msgd[0]>125) memset(msgd,0,sizeof(msgd));
  msgd_pos=strlen(msgd);

  EEPROM.get(EEPROM_MSG_HELL_ADR,msghl);
  if(msghl[0]<0 || msghl[0]>125) memset(msghl,0,sizeof(msghl));
  msghl_pos=strlen(msghl);

  EEPROM.get(EEPROM_MSG_JT9_ADR,msgjt);
  if(msgjt[0]<0 || msgjt[0]>125) memset(msgjt,0,sizeof(msgjt));
  msgjt_pos=strlen(msgjt);

  EEPROM.get(EEPROM_MSG_FST4_ADR,msgfst4);
  if(msgfst4[0]<0 || msgfst4[0]>125) memset(msgfst4,0,sizeof(msgfst4));
  msgfst4_pos=strlen(msgfst4);
  
 // EEPROM.get(EEPROM_MODE_ADR,mode);
 // if(mode<1)mode=1; if(mode>OPERATION_MODES)mode=OPERATION_MODES;

  EEPROM.get(EEPROM_CRYSTAL_ADR,Crystal);
  if(Crystal<0UL || Crystal>20000000UL)Crystal=12800000UL;
  for (byte i=0; i<CRYSTAL_LIST_N; i++) {if(Crystal==Crystal_list[i]) list_n=i;}

  set24bit();
  EEPROM.get(EEPROM_CODE_ADR,freq); if(isnan(freq) || freq<=1.0)freq=137500.0; //защита от nan при первом запуске
  Code=(unsigned long)(freq/Step);
  EEPROM.get(EEPROM_CODEQRSS_ADR,freqQRSS); if(isnan(freqQRSS) || freqQRSS<=1.0)freqQRSS=137710.0;
  CodeQRSS=(unsigned long)(freqQRSS/Step);
  EEPROM.get(EEPROM_CODEDFCW_ADR,freqDFCW); if(isnan(freqDFCW) || freqDFCW<=1.0)freqDFCW=137778.0;
  CodeDFCW=(unsigned long)(freqDFCW/Step);
  EEPROM.get(EEPROM_CODEHELL_ADR,freqHELL); if(isnan(freqHELL) || freqHELL<=1.0)freqHELL=137778.0;
  CodeHELL=(unsigned long)(freqHELL/Step);
  EEPROM.get(EEPROM_CODEWSPR_ADR,freqWSPR); if(isnan(freqWSPR) || freqWSPR<=1.0)freqWSPR=137570.0;
  CodeWSPR=(unsigned long)(freqWSPR/Step);
  EEPROM.get(EEPROM_CODEOP32_ADR,freqOP32); if(isnan(freqOP32) || freqOP32<=1.0)freqOP32=137511.0;
  CodeOP32=(unsigned long)(freqOP32/Step);
  EEPROM.get(EEPROM_CODEWSPR15_ADR,freqWSPR15);if(isnan(freqWSPR15) || freqWSPR15<=1.0)freqWSPR15=137615.0;
  CodeWSPR15=(unsigned long)(freqWSPR15/Step);
  EEPROM.get(EEPROM_CODEJT9_ADR,freqJT9);   if(isnan(freqJT9) || freqJT9<=1.0)freqJT9=137100.0;
  CodeJT9=(unsigned long)(freqJT9/Step);
  EEPROM.get(EEPROM_CODEFST4_ADR,freqFST4);   if(isnan(freqFST4) || freqFST4<=1.0)freqFST4=137500.0;
  CodeFST4=(unsigned long)(freqFST4/Step);
  EEPROM.get(EEPROM_CODEFW_ADR,freqFW);   if(isnan(freqFW) || freqFW<=1.0)freqFW=137500.0;
  CodeFW=(unsigned long)(freqFW/Step);
  EEPROM.get(EEPROM_CODECW_ADR,freqCW);   if(isnan(freqCW) || freqCW<=1.0)freqCW=137000.0;
  CodeCW=(unsigned long)(freqCW/Step);
  
  EEPROM.get(EEPROM_DOTQRSS_ADR,dot_qrss);
  if(dot_qrss>6000 || dot_qrss<0)dot_qrss=10;
  
  EEPROM.get(EEPROM_DOTDFCW_ADR,dot_dfcw);
  if(dot_dfcw>6000 || dot_dfcw<=0)dot_dfcw=90;

  EEPROM.get(EEPROM_DOTHELL_ADR,dot_hell);
  if(dot_hell>6000 || dot_hell<=0)dot_hell=90;

  EEPROM.get(EEPROM_DOTJT9_ADR,dot_jt9);
  if(dot_jt9>30 || dot_jt9<1)dot_jt9=10;
  for(byte i=0; i<5; i++) if(dot_jt9==jt9_list[i]) cnt_dot_jt9=i;

  EEPROM.get(EEPROM_DOTFST4_ADR,dot_fst4);
  if(dot_fst4>1800 || dot_fst4<15)dot_fst4=1800;
  for(byte i=0; i<7; i++) if(dot_fst4==fst4_list[i]) cnt_dot_fst4=i;

  EEPROM.get(EEPROM_DOTPAUSE_ADR,dot_pause);
  if(dot_pause>59 || dot_pause<0)dot_pause=0;
  
  EEPROM.get(EEPROM_BCNLIST_ADR,bcn_list);
  if(bcn_list[0]==(char)0xff) memset(bcn_list,0,sizeof(bcn_list));

  EEPROM.get(EEPROM_LF2_ADR,LF2);
  if(LF2==0xff)LF2=0;
  EEPROM.get(EEPROM_BORDERS_LF_ADR,borders_lf);
  if(borders_lf==0xff)borders_lf=1;
  
  EEPROM.get(EEPROM_BEACON_ADR,BEACON);
  if(BEACON==0xff)BEACON=0;

  EEPROM.get(EEPROM_TIMERCOR_ADR,timer_correction);
  if(isnan(timer_correction)) timer_correction=0;
  if(timer_correction>100 || timer_correction<-100){
    timer_correction=0;
  }
 EEPROM.get(EEPROM_TBEACONSTARTHH_ADR,t_bcn_start_hh);
 if(t_bcn_start_hh<0 || t_bcn_start_hh>23) t_bcn_start_hh=0;
 EEPROM.get(EEPROM_TBEACONSTARTMM_ADR,t_bcn_start_mm);
 if(t_bcn_start_mm<0 || t_bcn_start_mm>59) t_bcn_start_mm=0;
 EEPROM.get(EEPROM_TBEACONSTOPHH_ADR,t_bcn_stop_hh);
 if(t_bcn_stop_hh<0 || t_bcn_stop_hh>23) t_bcn_stop_hh=0;
 EEPROM.get(EEPROM_TBEACONSTOPMM_ADR,t_bcn_stop_mm);
 if(t_bcn_stop_mm<0 || t_bcn_stop_mm>59) t_bcn_stop_mm=0;
  
  delay(2000);
  clearLine(0);
  myOLED.clrScr();
  lcd.print(F("SEARCHING DDS..."));
  myOLED.print(F("SEARCHING DDS..."), CENTER, OL_DY);
  
  clearLine(1);
  if(BEACON){lcd.print(F("BEACON: ACTIVE"));myOLED.print(F("BEACON: ACTIVE"), CENTER, MY_OLED_STRING_HIGH+OL_DY);}
  myOLED.update();

  bool result=false;
 /* 
  if(analogRead(AD98XX_FLAG_PIN)<10) 
  {DDS_VER=0; DDSAD=true;
   set28bit();
   lcd.setCursor(0,0);lcd.print(F("DDS AD98xx FOUND"));
   myOLED.print(F("DDS AD98xx FOUND"), CENTER, MY_OLED_STRING_HIGH+OL_DY);myOLED.update();
   goto M4;
  }
 */ 
  for (int i=0; i<3; i++)
  { if(i==0)Serial3.write("H");else Serial3.write("?");
    result=readF();
    freq=Code*Step;
    if(result) break;
    delay(300);
  }
  if(result){
     clearLine(0);
     lcd.setCursor(3,0);lcd.print(F("DDS VER "));
     lcd.print(DDS_VER);
     myOLED.clrScr();
     myOLED.print(F("DDS VER "), CENTER, OL_DY);
     myOLED.printNumI(DDS_VER, CENTER, MY_OLED_STRING_HIGH+OL_DY);
     myOLED.update();
     delay(1000);
     if(DDS_VER>=3) {set32bit(); enable32();}
  }
  /*
  else
  {
    setgmdds();
    for(int i=0; i<3; i++)
    { 
     result=setF(Code,false);
     if(result) break;
     delay(300);
    }
    clearLine(0);myOLED.clrScr();
    if(result){lcd.setCursor(2,0);lcd.print(F("GM DDS FOUND")); myOLED.print(F("GM DDS FOUND"), CENTER, OL_DY);myOLED.update();delay(1000);}
  }
  */
  if(!result) {clearLine(0);lcd.print(F("DDS NOT DETECTED")); myOLED.print(F("DDS NOT DETECTED"), CENTER, OL_DY);myOLED.update();delay(2000);
               Serial3.begin(9600); Serial3.setTimeout(100);
               freq=137500.0; set24bit();
  }

M4:  delay(1000);
  if(LF2){lcd.setCursor(4,1);lcd.print(F("Fout x 2")); myOLED.print(F("Fout x 2"), CENTER, MY_OLED_STRING_HIGH+OL_DY);myOLED.update();delay(1000);}
  clearLine(0);clearLine(1);
  
  toneOFF();pttOFF();
  freq=137500.0;
  recomputeCode();
  setF(Code,false);
  time_of_settime=millis();
  if (rtc.isrunning()){
    DateTime now = rtc.now();
    t_set_hour=now.hour();
    t_set_min=now.minute();
    t_set_sec=now.second();
    setmytime(); //устанавливаем, если время правильное
  }
  mode=MODE_TIME;
  printMode(mode);
  if(detectButton()!=BTN_NONE) BEACON=0;
  if(BEACON){bcn_pos=0; start_beacon(0);}
}


unsigned long t_last_updw=0;
unsigned long t_last_tx=0;
unsigned int tx_pos_old=0;


bool waitToStart(char start_mode)
{ bool result=false;
  compute_time();
  if(start_mode==MODE_PAUSE){
   if(t_pause_millis<=millis()) goto MW;
   clearLine(0); lcd.print(F("PAUSE "));
   myOLED.print(F("PAUSE "),OL_DX,OL_DY);
   long pt=0;
   if(t_pause_millis>millis())pt=(t_pause_millis-millis())/1000;
   lcd.print(pt);lcd.print(F(" s"));
   myOLED.printNumI(pt,9*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY,4,' ');
   myOLED.print(F(" s"),13*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); 
  }
  
  if(!timeIsSet) return false; // точное время неизвестно! 
  
  if(start_mode==MODE_FST4 && dot_fst4<60) {
   if(t_sec%dot_fst4==0) result=true;
  }
  else if(t_sec!=0) return false;
  
  if(start_mode==MODE_WSPR) {
   if(t_min%2==0) result=true;
  }
  if(start_mode==MODE_WSPR15){
   if(t_min%15==0) result=true;
  }
  if(start_mode==MODE_JT9) {
   if(t_min%dot_jt9==0) result=true;
  }
   if((start_mode==MODE_FST4 || start_mode==MODE_FW) && dot_fst4>=60) {
   if(t_min%(dot_fst4/60)==0) result=true;
  }

  if(start_mode==MODE_T60){
   if(t_min==0) goto MW; 
  }
  if(start_mode==MODE_T30){
   if(t_min%30==0) goto MW; 
  }
  if(start_mode==MODE_TSTART){
    //if(t_bcn_start_hh==t_hour && t_bcn_start_mm>=t_min) goto MW;
    if(timeIsBetweenBcn()) goto MW;
  }
  if(!result) return false;
  mode=start_mode; printMode(mode); /// WSPR start
  mode=MODE_TX; mode_old=start_mode; tx_pos=0; delay(WSPR_PAUSE); t_last_tx=millis();timer1();paintTX();
  return true;
MW: 
  bcn_pos++; start_beacon(bcn_pos);
  return true;
}


void paintRotatingDash(void)
{
 static char d[]=": ";
 static byte cnt;
 static uint32_t last_dash;
 if(millis()-last_dash<500)return;
 last_dash=millis();
 lcd.setCursor(10,0);
 lcd.print(d[cnt]);     
 if(cnt)myOLED.print(":",10*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
 else myOLED.print(" ",10*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
 myOLED.update();
 cnt++;cnt=cnt%2;;
}

void printFst4Menu(byte b)
{ lcd.setCursor(11,0);
  switch(b)
        { case FST_0: lcd.print(F("0    ")); myOLED.print(F("0    "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_1: lcd.print(F("1    ")); myOLED.print(F("1    "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_2: lcd.print(F("2    ")); myOLED.print(F("2    "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_3: lcd.print(F("3    ")); myOLED.print(F("3    "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_DEL: lcd.print(F("<-del")); myOLED.print(F("<-del"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_SAVE: lcd.print(F("SAVE ")); myOLED.print(F("SAVE "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_LOAD: lcd.print(F("LOAD ")); myOLED.print(F("LOAD "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_ERASE: lcd.print(F("ERASE")); myOLED.print(F("ERASE"),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case FST_GEN: lcd.print(F("MAKE ")); myOLED.print(F("MAKE "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          default: ;
        }
  myOLED.update();
}


void printBcnMenu(byte b)
{ lcd.setCursor(7,0);
  switch(b)
        { case BCN_LOAD: lcd.print(F("load ")); myOLED.print(F("load "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_RUN : lcd.print(F("run  ")); myOLED.print(F("run  "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_QRSS: lcd.print(F("QRSS ")); myOLED.print(F("QRSS "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_DFCW: lcd.print(F("DFCW ")); myOLED.print(F("DFCW "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_HELL: lcd.print(F("HELL ")); myOLED.print(F("HELL "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_OP32: lcd.print(F("OP32 ")); myOLED.print(F("OP32 "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_WSPR: lcd.print(F("WSPR ")); myOLED.print(F("WSPR "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_WSPR15: lcd.print(F("WR15 ")); myOLED.print(F("WR15 "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_JT9: lcd.print(F("JT-9 ")); myOLED.print(F("JT-9 "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          /*
          case BCN_FST4: EEPROM.get(EEPROM_DOTFST4_ADR,dot_fst4); if(dot_fst4>=1800 || dot_fst4<120)dot_fst4=120;
                         if(dot_fst4<=120||dot_fst4==1800){lcd.print(F("FW-2")); myOLED.print(F("FW-2"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                         if(dot_fst4==300){lcd.print(F("FW-5")); myOLED.print(F("FW-5"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                         if(dot_fst4==900){lcd.print(F("FW15")); myOLED.print(F("FW15"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);}
                        break;
          */
          case BCN_FW2: lcd.print(F("FW-2   ")); myOLED.print(F("FW-2   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;              
          case BCN_FW5: lcd.print(F("FW-5   ")); myOLED.print(F("FW-5   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_FW15: lcd.print(F("FW15   ")); myOLED.print(F("FW15   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_FW30: lcd.print(F("FW30   ")); myOLED.print(F("FW30   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_RPT: lcd.print(F("RPT    ")); myOLED.print(F("RPT    "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_SAVE: lcd.print(F("save   ")); myOLED.print(F("save   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_CLR: lcd.print(F("clr    ")); myOLED.print(F("clr    "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_T60: lcd.print(F("T=00   ")); myOLED.print(F("T=00   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_T30: lcd.print(F("T=30   ")); myOLED.print(F("T=30   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_TSTART: lcd.print(F("T=hh:mm")); myOLED.print(F("T=hh:mm"),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_NO_TSTART: lcd.print(F("T=now  ")); myOLED.print(F("T=now  "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          case BCN_PAUSE: lcd.print (F("PAUSE  ")); myOLED.print(F("PAUSE   "),7*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);break;
          default: ;
        }
  myOLED.update();
}
//////////////////////////////////////////////////////////////////////
void start_beacon(byte pos)
{
M3:
 if(pos>=strlen(bcn_list)){ //end of schedule
  mode=mode_old; toneOFF(); pttOFF(); tx_pos=0; tx_len=0;
  stop_beacon();
  return;
 }
 switch(bcn_list[pos])
 { case 'Q': printMode(MODE_QRSS);start_qrss(); break;
   case 'D': printMode(MODE_DFCW);start_dfcw(); break;
   case 'H': printMode(MODE_HELL);start_hell(); break;
   case 'O': printMode(MODE_OP32);start_op32(); break; 
   case 'w': printMode(MODE_WSPR);start_wspr(); break;
   case 'W': printMode(MODE_WSPR15);start_wspr15(); break;
   case 'J': printMode(MODE_JT9);start_jt9(); break;
   //case 'f': printMode(MODE_FW);
   //          EEPROM.get(EEPROM_DOTFST4_ADR,dot_fst4); if(dot_fst4>=1800 || dot_fst4<120)dot_fst4=120;
   //          start_fst4(1); break;
   case '2': printMode(MODE_FW);dot_fst4=120;start_fst4(1); break;
   case '5': printMode(MODE_FW);dot_fst4=300;start_fst4(1); break;
   case 'f': printMode(MODE_FW);dot_fst4=900;start_fst4(1); break;           
   case 'F': printMode(MODE_FW);dot_fst4=1800;start_fst4(1); break;
   case 'R': bcn_pos=0; pos=0; goto M3; break;
   case 'T': mode_start=MODE_T60; goto MT; break;
   case 't': mode_start=MODE_T30; goto MT; break;
   case '=': mode_start=MODE_TSTART; goto MT; break;
   case 'P': mode_start=MODE_PAUSE; t_pause_millis=(millis() + dot_pause*60000);
             goto MT; break;
   default: return;
 }
 return;
 MT:
  printMode(mode_start); 
  mode=mode_start; mode_old=MODE_BEACON; mode_next=MODE_TX;
 return;
}

void stop_beacon(void){
  BEACON=false; bcn_pos=0;
  EEPROM.put(EEPROM_BEACON_ADR,BEACON);
  EEPROM.get(EEPROM_DOTFST4_ADR,dot_fst4); //if(dot_fst4>1800 || dot_fst4<120)dot_fst4=120;
}
//////////////////////////////////////////////////////////////////////
void start_qrss(void)
{      //init_dds();
       freq=Step*CodeQRSS; freqQRSS=freq;
       if(DDS_VER>=3){set32bit(); enable32();}
       //Code=CodeQRSS;
       setF(CodeQRSS, false);
       EEPROM.put(EEPROM_CODEQRSS_ADR,freqQRSS);
       tx_len=qrss_coding(msgq);
       tx_pos=0; tx_dc=0; mode_old=MODE_QRSS;mode=MODE_TX;
       mode_start=MODE_QRSS;
       timer_interval=1000UL * dot_qrss;
       if(timer_interval<=0)timer_interval=150UL;
       t_last_tx=millis();timer1();
       paintTX();
}
void start_dfcw(void)
{      //init_dds();
       freq=Step*CodeDFCW; freqDFCW=freq;
       if(DDS_VER>=3){set32bit(); enable32();}
       //Code=CodeDFCW;
       setF(CodeDFCW, false);
       EEPROM.put(EEPROM_CODEDFCW_ADR,freqDFCW);
       tx_len=dfcw_coding(msgd);
       tx_pos=0;
       timer_interval=dot_dfcw*333UL;
       double tx_df=3000.0/timer_interval; // 4500.0
       if(dot_dfcw>=60)tx_df=tx_df*dot_dfcw/60.0;
       tx_dc=tx_df/Step;
       if(tx_dc<1)tx_dc=1;       
       mode_old=MODE_DFCW;mode=MODE_TX;mode_start=MODE_DFCW;
       t_last_tx=millis();timer1();
       paintTX();
}
void start_hell(void)
{      //init_dds();
       freq=Step*CodeHELL; freqHELL=freq;
       if(DDS_VER>=3){set32bit(); enable32();}
       setF(CodeHELL, false);
       EEPROM.put(EEPROM_CODEHELL_ADR,freqHELL);
       tx_len=hell_coding(msghl);
       tx_pos=0;
       timer_interval=dot_hell*333UL; //283
       double tx_df=1200.0/timer_interval; //1500.0/timer_interval;
       if(dot_hell>=60)tx_df=tx_df*dot_hell/60.0;
       tx_dc=tx_df/Step;
       if(tx_dc<1)tx_dc=1;       
       mode_old=MODE_HELL;mode=MODE_TX;mode_start=MODE_HELL;
       t_last_tx=millis();timer1();
       paintTX();
}

void start_op32(void)
{     //init_dds();
       freq=Step*CodeOP32; freqOP32=freq;
       if(DDS_VER>=3){set32bit(); enable32();}
       //Code=CodeOP32;
       setF(CodeOP32, false);
       EEPROM.put(EEPROM_CODEOP32_ADR,freqOP32);
       tx_len=opera_coding();tx_pos=0;
       tx_dc=0;
       timer_interval=8192UL;  
       mode_old=MODE_OP32; mode=MODE_TX;mode_start=MODE_OP32;
       t_last_tx=millis();timer1();
       paintTX();
}
void start_wspr(void)
{      //init_dds();
       freq=Step*CodeWSPR; freqWSPR=freq;
       if(DDS_VER>0 && DDS_VER<3){set24bit(); enable24();}
       if(DDS_VER>=3){set32bit(); enable32();}
       //Code=CodeWSPR;
       setF(CodeWSPR, false);
       EEPROM.put(EEPROM_CODEWSPR_ADR,freqWSPR);
       prepare_wspr(1);
       mode_start=MODE_WSPR;mode_next=MODE_TX; mode=MODE_WAIT;
}
void prepare_wspr(byte mul)
{
       tx_len=wspr_coding();
       tx_pos=0;timer_interval=683UL * mul;
       double tx_df=1.464844 / double(mul);
       double df1=tx_df/Step;
       tx_dc=tx_df/Step;
       if(df1-double(tx_dc)>=0.5) tx_dc=tx_dc+1;
       if(tx_dc<1)tx_dc=1;
        // для старой версии записать четыре частоты в DDS
       if(DDS_VER>0 && DDS_VER<4 && !BIT32)
       {unsigned long cd4[4];
        cd4[0]=Code;
        cd4[1]=Code + (unsigned long)(df1);    if(df1-double(cd4[1]-Code)>=0.5) cd4[1]++;   if(cd4[1]<=cd4[0])cd4[1]=cd4[0]+1UL;
        cd4[2]=Code + (unsigned long)(2.0*df1);if(2.0*df1-double(cd4[2]-Code)>=0.5) cd4[2]++; if(cd4[2]<=cd4[1])cd4[2]=cd4[1]+1UL;
        cd4[3]=Code + (unsigned long)(3.0*df1);if(3.0*df1-double(cd4[3]-Code)>=0.5) cd4[3]++; if(cd4[3]<=cd4[2])cd4[3]=cd4[2]+1UL;
        setF4(cd4);
       }
  
}
void start_wspr15(void)
{     //init_dds();
      freq=Step*CodeWSPR15; freqWSPR15=freq;
      if(DDS_VER>0 && DDS_VER<3){set24bit(); enable24();}
      if(DDS_VER>=3){set32bit(); enable32();}
      //Code=CodeWSPR15;
      setF(CodeWSPR15, false);
      EEPROM.put(EEPROM_CODEWSPR15_ADR,freqWSPR15);
      prepare_wspr(8);
      mode_start=MODE_WSPR15;mode_next=MODE_TX; mode=MODE_WAIT;
}
void start_jt9(void)
{     //init_dds();
      freq=Step*CodeJT9; freqJT9=freq;
      if(DDS_VER>=3){set32bit(); enable32();}
      setF(CodeJT9, false);
      EEPROM.put(EEPROM_CODEJT9_ADR,freqJT9); 
      tx_len=jt9_coding(msgjt);
      tx_pos=0;
       timer_interval=576;
       double tx_df=1.736;
       switch(dot_jt9){
        case 2:  tx_df=1200.0/1536.0;     timer_interval=1280; break;
        case 5:  tx_df=1200.0/4096.0;     timer_interval=3413; break;
        case 10: tx_df=12000.0/82944.0;   timer_interval=6912; break;
        case 30: tx_df=12.0/252.0;        timer_interval=21000; break;
       }
       tx_dc=tx_df/Step;
       if(tx_dc<1)tx_dc=1;
      mode_start=MODE_JT9;mode_next=MODE_TX; mode=MODE_WAIT;
 //Serial.println(tx);
}

void start_fst4(bool isW)
{     //init_dds();
      freq=Step*CodeFST4; freqFST4=freq;
      if(DDS_VER>0 && DDS_VER<3){set24bit(); enable24();} // здесь надо исследовать
      if(DDS_VER>=3){set32bit(); enable32();} // была только эта строка и >=3
      setF(CodeFST4, false);
      EEPROM.put(EEPROM_CODEFST4_ADR,freqFST4);
      if(isW) { fst4w_init_beacon(call, qth, pwr); if(dot_fst4<120)dot_fst4=120;}
      else tx_len=fst4_coding(msgfst4);
      tx_pos=0;
       timer_interval=683UL;
       double tx_df=1.464844;
       switch(dot_fst4){
        case 15:  tx_df=16.667;  timer_interval=60UL; break;
        case 30:  tx_df=7.1428;   timer_interval=140UL; break;
        case 60:  tx_df=3.0864;   timer_interval=324UL; break;
        case 120: tx_df=1.464;   timer_interval=683UL; break;
        case 300:  tx_df=0.558;  timer_interval=1792UL; break;
        case 900:  tx_df=0.18028;  timer_interval=5547UL; break;
        case 1800: tx_df=0.089285; timer_interval=11200UL; break;
       }
       tx_dc=tx_df/Step;
       if(tx_dc<1)tx_dc=1;
       double df1=tx_df/Step;
        // для старой версии записать четыре частоты в DDS
       if(DDS_VER>0 && DDS_VER<4 && !BIT32)
       {unsigned long cd4[4];
        cd4[0]=Code;
        cd4[1]=Code + (unsigned long)(df1);    if(df1-double(cd4[1]-Code)>=0.5) cd4[1]++;   if(cd4[1]<=cd4[0])cd4[1]=cd4[0]+1UL;
        cd4[2]=Code + (unsigned long)(2.0*df1);if(2.0*df1-double(cd4[2]-Code)>=0.5) cd4[2]++; if(cd4[2]<=cd4[1])cd4[2]=cd4[1]+1UL;
        cd4[3]=Code + (unsigned long)(3.0*df1);if(3.0*df1-double(cd4[3]-Code)>=0.5) cd4[3]++; if(cd4[3]<=cd4[2])cd4[3]=cd4[2]+1UL;
        setF4(cd4);
       }
      if(isW)mode_start=MODE_FW;else mode_start=MODE_FST4;
      mode_next=MODE_TX; mode=MODE_WAIT;
      //Serial.print(tx);
}

bool timeIsBetweenBcn(void)
{
  if(t_hour==t_bcn_start_hh && t_min==t_bcn_start_mm) {return true;}
  int start=(int)t_bcn_start_hh*60+t_bcn_start_mm;
  int stopt=(int)t_bcn_stop_hh*60+t_bcn_stop_mm;
  int nw=(int)t_hour*60+t_min;
  if(stopt>=start && nw>=start && nw<stopt){return true;} //нет перехода через полночь
  if(stopt<=start && (nw>=start || nw<stopt)){return true;} // с переходом через полночь
  return false;
}

///////////////////////////////////////////////////////////////////////
#define DREBEZG 50 //ms
#define ANALOG_ZERO 100 //порог ниже которого аналоговый вход считается соединенным с землей
#define QSK_TIME 1000 //время QSK - через сколько мс отключится PTT по окончании нажатия CW

void loop() {
  // put your main code here, to run repeatedly
static unsigned long last_time_print;
if(mode==MODE_TX && (millis()-t_last_tx >= timer_interval*tx_pos - timer_correction*timer_interval*tx_pos/1000.0)){timer1();}
if(BEACON && mode!=MODE_TSTART &&t_sec%10==0){ //если достигнуто время окончания передачи 
 if(timeIsSet &&!timeIsBetweenBcn()){
  if(bcn_list[0]=='=') {toneOFF(); pttOFF(); tx_pos=0; tx_len=0; BEACON=true; bcn_pos=0; start_beacon(0);} //если режим старта по времени включен, то начинаем сначала
 }
}
 //////////// GPS - MEGA ///////////////
 #ifdef BOARD_MEGA
  if(gps_on){
    while(Serial2.available())gps_read();
  }
 #endif
#ifdef GPS_DEBUG
  while(Serial.available()) {int c=Serial.read(); Serial2.write(c);}
#endif 
 /*// сквозная трансляция от компьютера к синтезатору для плат старого образца (standalone)
 while(Serial1.available()) {int c=Serial1.read(); Serial3.write(c);
                             mode=MODE_EXTERNAL_SYNT; submenu=true;
 }*/

 //// сквозная трансляция от коспьютера к синтезатору
 if(mode==MODE_EXTERNAL_SYNT){
  while(Serial.available()) {byte c1=Serial.read(); Serial3.write(c1); lcd.setCursor(5,1);lcd.print(c1,HEX);/*myOLED.printNumI(c1,5*MY_OLED_CHAR_WIDTH,MY_OLED_STRING_HIGH);myOLED.update();*/}
  while(Serial3.available()) {byte c2=Serial3.read(); Serial.write(c2);lcd.setCursor(14,1);lcd.print(c2,HEX);/*myOLED.printNumI(c2,14*MY_OLED_CHAR_WIDTH,MY_OLED_STRING_HIGH);myOLED.update();*/}
 }
 ///////////////////////////////////////

 //// сквозная трансляция от коспьютера к GPS
 if(mode==MODE_EXTERNAL_GPS){
  while(Serial.available()) {char c1[2];c1[1]=0;c1[0]=Serial.read(); Serial2.write(c1[0]);lcd.setCursor(5,1);lcd.print(c1);/*myOLED.print(c1,5*MY_OLED_CHAR_WIDTH,MY_OLED_STRING_HIGH);myOLED.update();*/}
  while(Serial2.available()){char c2[2];c2[1]=0;c2[0]=Serial2.read(); Serial.write(c2[0]);lcd.setCursor(14,1);lcd.print(c2);/*myOLED.print(c2,14*MY_OLED_CHAR_WIDTH,MY_OLED_STRING_HIGH);myOLED.update();*/}
 }
 ///////////////////////////////////////

//// загрузка тонов для FST4
//// от компьютера ожидается строка: Z031231230010310231023012001201230102023...., Z - признак начала записи
if(mode==MODE_LOAD_FST4){
  while(Serial.available()) {
    byte c1=Serial.read();
    if(c1=='Z'){ ///// флаг начала записи
      memset(msgfst4,0,sizeof(msgfst4));
      msgfst4_pos=0;
      Serial.write('Z');
    }
    if(msgfst4_pos<MSG_FST4_LEN_MAX && (c1=='0' || c1=='1' || c1=='2' || c1=='3')){
      msgfst4[msgfst4_pos]=c1;msgfst4_pos++;
      printFst4msg();
      Serial.write(c1);
    }
  }
}
////////////////////////////////////////
 
 if(mode!=MODE_EXTERNAL_GPS && mode!=MODE_EXTERNAL_SYNT && mode!=MODE_LOAD_FST4) {
  if(Serial.available()) {char c3=Serial.read();
                          if(c3=='?') {mode=MODE_EXTERNAL_SYNT; submenu=true; printMode(mode);Serial3.write('?');}
  }
 }

 if(mode==MODE_GPS){
  static unsigned long last_gps_print;
  if(millis()-last_gps_print>=250UL){
    last_gps_print=millis();
    printGPS();
  }
 }

 if(mode>=MODE_WAIT && mode_next==MODE_TX)  waitToStart(mode_start);
 
 if(millis()-last_time_print>=250UL){
  //printVbat(); 
  last_time_print=millis();
  compute_time();
  if(mode>=MODE_WAIT) {printTime(t_hour,t_min,t_sec);
    //if(mode_next==MODE_TX)  waitToStart(mode_start);
  }
  if(mode==MODE_TX && timer_interval>1000)paintRotatingDash();
  if(mode==MODE_TIME && timeIsSet)printTime(t_set_hour,t_min,t_sec);
  if(mode==MODE_QTH){
   lcd.setCursor(1,1);
   if(gps_ready) {lcd.print(F("GPS=")); lcd.print(qth); gps_ready=false; 
                  myOLED.print(F("GPS="), MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
                  myOLED.print(qth, 7*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
                 // myOLED.update();
                 }
   else {if(gps_detected) {lcd.print(F("GPS "));gps_detected=false;
                           myOLED.print(F("GPS "), MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
                           //myOLED.update();
                          }
         else  {lcd.print(F("    ")); myOLED.print(F("    "), MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);/*myOLED.update();*/}
        }
  }
 }
 
 if(Code!=Code_old){freq=Step*Code; printFreq(Code);Code_old=Code;} 
 if(mode==MODE_TX && (tx_pos!=tx_pos_old)){paintTX(); tx_pos_old=tx_pos;}
 if(mode!=MODE_TX && mode!=mode_old){printMode(mode);mode_old=mode;}
 
 if(mode==MODE_CTRL)
 { static unsigned long last_tone;
   if(IsToneOn) last_tone=millis();
   int ua1=analogRead(A1);
   if(ua1<ANALOG_ZERO){
      //if(button==BTN_SELECT){}//в субменю CW
      if(IsToneOn==0){if(!IsPttOn)pttON(); toneON();  printTonePtt();}
   }
   else {
      if(IsToneOn==1){toneOFF(); printTonePtt();}
      if(IsPttOn && (millis()-last_tone>QSK_TIME)){pttOFF();printTonePtt();}
   };
 }
 
 byte button = detectButton();
 static unsigned long tb, tb1;
 if(millis()-tb1<DREBEZG) button=btn_old;
 else tb1=millis();
 if(button==btn_old && (mode<=OPERATION_MODES || mode==MODE_TIME || mode==MODE_TIMER_CORRECTION || mode==MODE_T_BEACON_START || mode==MODE_T_BEACON_STOP) && (button==BTN_UP || button==BTN_DOWN)) //автоповтор
 {
  if(millis()-t_last_updw>200UL){t_last_updw=millis();btn_old=BTN_NONE;}
 }
 /*
 if(button==btn_old && (mode==MODE_TIMER_CORRECTION) && (button==BTN_UP || button==BTN_DOWN)) //автоповтор
 {
  if(millis()-t_last_updw>100UL){t_last_updw=millis();btn_old=BTN_NONE;}
 }
 */
 if(button!=btn_old && (millis()-tb>DREBEZG))
 {tb=millis();
  if(mode==MODE_TX)
  { if(button==BTN_SELECT)
    {   toneOFF(); pttOFF();
        tx_pos=0;tx_len=0;
        stop_beacon();
         mode=mode_old; mode_old=MODE_TX;
         printMode(mode);
    }
  }
  else if(mode>=MODE_WAIT && mode_next==MODE_TX && button==BTN_SELECT)
    {if(mode==MODE_WAIT)mode=mode_start;//чтобы можно было отменить ожидание WSPR
     else mode=MODE_BEACON;
     stop_beacon();
     printMode(mode);
    }
  else if(mode!=MODE_WAIT && mode!=MODE_TSTART && mode!=MODE_T60 && mode!=MODE_T30)
  switch (button) {
    case BTN_UP:
      if(mode<=OPERATION_MODES)
      { 
        updwdc(mode, BTN_UP);
      }
      if(mode==MODE_MSG_QRSS || mode==MODE_MSG_DFCW || mode==MODE_MSG_JT9 || mode==MODE_MSG_HELL)
      {
        m[m_pos]='-';
        msg_updw(); 
      }
      if(mode==MODE_CALL)
      { cc[cc_pos]='-';
        call_updw();
      }
      if(mode==MODE_QTH)
      { qq[qq_pos]='-';
        lcd.setCursor(11,0);
        lcd.print(qq);
        myOLED.print(qq, 11*MY_OLED_CHAR_WIDTH+OL_DX, /*MY_OLED_STRING_HIGH+*/OL_DY);
        myOLED.update();
        qq_pos++; if(qq_pos>=QQ_LEN_MAX-1)qq_pos=QQ_LEN_MAX-1;
      }
      if(mode==MODE_PWR)
      { n_pwr++; if(n_pwr>18)n_pwr=18;
        power_updw();
      }
      if(mode==MODE_CLR_EEPROM)
      {
        clr_eeprom_updw();
      }
      if(mode==MODE_GEN)
      {
        list_n++;
        if(list_n>=CRYSTAL_LIST_N)list_n=0;//CRYSTAL_LIST_N-1;
        gen_updw(); 
      }
      if(mode==MODE_DOT_QRSS)
      {
        if(dot_qrss<10)dot_qrss++; 
        else {
           if(dot_qrss<120)dot_qrss=dot_qrss+10;
           else dot_qrss=dot_qrss+60;
        }
        if(dot_qrss>600)dot_qrss=600;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
        myOLED.printNumI(dot_qrss, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_PAUSE)
      {
        dot_pause++; 
        if(dot_pause>59)dot_pause=59;
        clearLine(1);lcd.setCursor(11,1); lcd.print(dot_pause);lcd.print(F("min"));
        myOLED.printNumI(dot_pause, 11*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("min"), 14*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_DFCW)
      {
        if(dot_dfcw<10)dot_dfcw++; 
        else {
          if(dot_dfcw<120)dot_dfcw=dot_dfcw+10;
          else dot_dfcw=dot_dfcw+60;
        }
        if(dot_dfcw>600)dot_dfcw=600;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_dfcw);lcd.print(F("s"));
        myOLED.printNumI(dot_dfcw, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_HELL)
      {
        if(dot_hell<10)dot_hell++; 
        else {
          if(dot_hell<120)dot_hell=dot_hell+10;
          else dot_hell=dot_hell+60;
        }
        if(dot_hell>600)dot_hell=600;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_hell);lcd.print(F("s"));
        myOLED.printNumI(dot_hell, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_JT9)
      { cnt_dot_jt9++; if(cnt_dot_jt9>=4) cnt_dot_jt9=4;
        dotjt9updw();
      }
      if(mode==MODE_DOT_FST4)
      { cnt_dot_fst4++; if(cnt_dot_fst4>=6) cnt_dot_fst4=6;
        dotfst4updw();
      }
      if(mode==MODE_TIME && !timeIsSet)
      {  //t_set_min++; 
         //if(t_set_min==60) t_set_hour++;
         //t_set_min=t_set_min%60; t_set_hour=t_set_hour%24;
         t_set_hour++;t_set_hour=t_set_hour%24;
         t_set_sec=0;
         printTime(t_set_hour,t_set_min,t_set_sec);
      }
      if(mode==MODE_T_BEACON_START)
      { t_bcn_start_hh++;
        t_bcn_start_hh=t_bcn_start_hh%24;
        char s[3]; sprintf(s,"%02d",t_bcn_start_hh);
        lcd.setCursor(10,0); lcd.print(s);myOLED.print(s,10*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
        myOLED.update();
      }
      if(mode==MODE_T_BEACON_STOP)
      { t_bcn_stop_hh++;
        t_bcn_stop_hh=t_bcn_stop_hh%24;
        char s[3]; sprintf(s,"%02d",t_bcn_stop_hh);
        lcd.setCursor(10,1); lcd.print(s);myOLED.print(s,10*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_BEACON)
      { 
        bcn++; bcn=bcn%BCN_MAX;
        printBcnMenu(bcn);
      }
      if(mode==MODE_MSG_FST4)
      { 
        fst++; fst=fst%FST_MAX;
        printFst4Menu(fst);
      }
      if(mode==MODE_LF2)
      {
        if(LF2)LF2=0;else LF2=1; lf2updw();
      }
      if(mode==MODE_BORDERS_LF)
      {
        if(borders_lf)borders_lf=0;else borders_lf=1; borderslfupdw();
      }
      if(mode==MODE_TIMER_CORRECTION)
      {
        timer_correction=timer_correction+0.1/6;
        timcorupdw();
      }
      last_btn=button;
      break;
    case BTN_DOWN:
      if(mode<=OPERATION_MODES)
      {
       updwdc(mode, BTN_DOWN);
      }
      if(mode==MODE_MSG_QRSS || mode==MODE_MSG_DFCW || mode==MODE_MSG_JT9 || mode==MODE_MSG_HELL)
      { m[m_pos]='.';
        msg_updw();
      }
      if(mode==MODE_CALL)
      { cc[cc_pos]='.';
        call_updw();
      }
      if(mode==MODE_QTH)
      { qq[qq_pos]='.';
        lcd.setCursor(11,0);
        lcd.print(qq);
        myOLED.print(qq, 11*MY_OLED_CHAR_WIDTH+OL_DX, /*MY_OLED_STRING_HIGH+*/OL_DY);
        myOLED.update();
        qq_pos++; if(qq_pos>=QQ_LEN_MAX-1)qq_pos=QQ_LEN_MAX-1;
      }
      if(mode==MODE_PWR)
      { n_pwr--; if(n_pwr<=0)n_pwr=0;
        power_updw();
      }
      if(mode==MODE_CLR_EEPROM){ clr_eeprom_updw();}
      if(mode==MODE_GEN)
      { if(list_n==0) list_n=CRYSTAL_LIST_N-1; else list_n--;
        gen_updw();
      }
      if(mode==MODE_DOT_QRSS)
      {
        if(dot_qrss<=10)dot_qrss--;
        else if(dot_qrss<=120)dot_qrss=dot_qrss-10;
             else dot_qrss=dot_qrss-60;
        if(dot_qrss<=0)dot_qrss=0;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
        myOLED.printNumI(dot_qrss, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_PAUSE)
      {
        dot_pause--; 
        if(dot_pause<0)dot_pause=0;
        clearLine(1);lcd.setCursor(11,1); lcd.print(dot_pause);lcd.print(F("min"));
        myOLED.printNumI(dot_pause, 11*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("min"), 14*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_DFCW)
      {
        if(dot_dfcw<=10)dot_dfcw--; 
        else if(dot_dfcw<=120)dot_dfcw=dot_dfcw-10;
             else dot_dfcw=dot_dfcw-60;
        if(dot_dfcw<=1)dot_dfcw=1;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_dfcw);/*lcd.setCursor(15,1);*/lcd.print(F("s"));
        myOLED.printNumI(dot_dfcw, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_HELL)
      {
        if(dot_hell<=10)dot_hell--; 
        else if(dot_hell<=120)dot_hell=dot_hell-10;
             else dot_hell=dot_hell-60;
        if(dot_hell<=1)dot_hell=1;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_hell);/*lcd.setCursor(15,1);*/lcd.print(F("s"));
        myOLED.printNumI(dot_hell, 12*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.print(F("s"), 15*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_JT9)
      { cnt_dot_jt9--; if(cnt_dot_jt9<0) cnt_dot_jt9=0;
        dotjt9updw();
      }
      if(mode==MODE_DOT_FST4)
      { cnt_dot_fst4--; if(cnt_dot_fst4<0) cnt_dot_fst4=0;
        dotfst4updw();
      }
      if(mode==MODE_TIME && !timeIsSet)
      {  //t_set_min--;if(t_set_min<0){t_set_min=59;t_set_hour--;} 
         //if(t_set_hour<0)t_set_hour=23;
         t_set_min++;
         t_set_min=t_set_min%60;t_set_hour=t_set_hour%24;
         t_set_sec=0;
         printTime(t_set_hour,t_set_min,t_set_sec);
      }
      if(mode==MODE_T_BEACON_START)
      { t_bcn_start_mm++;
        t_bcn_start_mm=t_bcn_start_mm%60; 
        char s[3]; sprintf(s,"%02d",t_bcn_start_mm);
        lcd.setCursor(13,0); lcd.print(s);myOLED.print(s,13*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
        myOLED.update();
      }
      if(mode==MODE_T_BEACON_STOP)
      { t_bcn_stop_mm++;
        t_bcn_stop_mm=t_bcn_stop_mm%60; 
        char s[3]; sprintf(s,"%02d",t_bcn_stop_mm);
        lcd.setCursor(13,1); lcd.print(s);myOLED.print(s,13*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_BEACON)
      { 
        if(bcn==0) bcn=BCN_MAX-1; else bcn--; 
        bcn=bcn%BCN_MAX;
        printBcnMenu(bcn);
      }
      if(mode==MODE_MSG_FST4)
      { 
        if(fst==0) fst=FST_MAX-1; else fst--; 
        fst=fst%FST_MAX;
        printFst4Menu(fst);
      }
      if(mode==MODE_LF2)
      {
        if(LF2)LF2=0;else LF2=1; lf2updw();
      }
      if(mode==MODE_BORDERS_LF)
      {
        if(borders_lf)borders_lf=0;else borders_lf=1; borderslfupdw();
      }
      if(mode==MODE_TIMER_CORRECTION)
      {
        timer_correction=timer_correction-0.1/6;
        timcorupdw();
      }
      last_btn=button;
      break;
    case BTN_LEFT:
      if(mode==MODE_CTRL){toneOFF();pttOFF();}
      mode=mode-1;
      if(!submenu && mode<=0)mode=MODE_SUBMENU;
      if(submenu && mode<MODE_SUBMENU) mode=MODE_MAX-1;
      last_btn=button;
      break;
    case BTN_RIGHT:
      if(mode==MODE_CTRL){toneOFF();pttOFF();}
      mode=mode+1;
      if(submenu && mode>=MODE_MAX) mode=MODE_SUBMENU;
      if(!submenu && mode>MODE_SUBMENU)mode=1;
      last_btn=button;
      break;
    case BTN_SELECT:    
      if(mode==MODE_CTRL)
      {//freq=Step*Code;
       //setF(Code, false);
      // EEPROM.put(EEPROM_CODE_ADR,freq);
       freqCW=Step*CodeCW;
       setF(CodeCW, false);
       EEPROM.put(EEPROM_CODECW_ADR,freqCW);
      }
      if(mode==MODE_QRSS)
      {
       start_qrss();
      }
      if(mode==MODE_DFCW)
      {
       start_dfcw();
      }
      if(mode==MODE_HELL)
      {
       start_hell();
      }
      if(mode==MODE_WSPR)
      {
         start_wspr();
      }
      if(mode==MODE_WSPR15)
      {
       start_wspr15();
      }
      if(mode==MODE_OP32)
      {
       start_op32();
      }
      if(mode==MODE_JT9)
      {
       start_jt9();
      }
      if(mode==MODE_FST4)
      {
       start_fst4(0);
      }
      if(mode==MODE_FW)
      {
        //fst4w_init_beacon(call, qth, pwr);
        start_fst4(1);
      }
      if(mode==MODE_MSG_QRSS)
      { clearLine(0); lcd.print(F("MSG QRSS:"));
        myOLED.print(F("MSG QRSS:"), OL_DX, OL_DY);
        myOLED.update();
        msg_select(msgq,msgq_pos);
        EEPROM.put(EEPROM_MSG_QRSS_ADR,msgq);
      }
      if(mode==MODE_MSG_DFCW)
      { clearLine(0); lcd.print(F("MSG DFCW:"));
        myOLED.print(F("MSG DFCW:"), OL_DX, OL_DY);
        myOLED.update();
        msg_select(msgd,msgd_pos);
        EEPROM.put(EEPROM_MSG_DFCW_ADR,msgd);
      }
      if(mode==MODE_MSG_HELL)
      { clearLine(0); lcd.print(F("MSG HELL:"));
        myOLED.print(F("MSG HELL:"), OL_DX, OL_DY);
        myOLED.update();
        msg_select(msghl,msghl_pos);
        EEPROM.put(EEPROM_MSG_HELL_ADR,msghl);
      }
      if(mode==MODE_MSG_JT9)
      { clearLine(0); lcd.print(F("MSG JT-9:"));
        myOLED.print(F("MSG JT-9:"), OL_DX, OL_DY);
        myOLED.update();
        msg_select(msgjt,msgjt_pos); msgjt[13]=0;
        EEPROM.put(EEPROM_MSG_JT9_ADR,msgjt);
      }
      if(mode==MODE_MSG_FST4)
      { 
        byte len=strlen(msgfst4);
        if(len>=MSG_FST4_LEN_MAX && fst>=FST_0 && fst<=FST_3)msgfst4[MSG_FST4_LEN_MAX-1]=0; 
        switch(fst)
        { case FST_LOAD:memset(msgfst4,0,sizeof(msgfst4)); 
                        EEPROM.get(EEPROM_MSG_FST4_ADR,msgfst4);
                        if(msgfst4[0]<'0'||msgfst4[0]>'3') memset(msgfst4,0,sizeof(msgfst4)); 
                        msgfst4_pos=strlen(msgfst4);//или +1 ????????????? 
                        break;
          case FST_0:   strcat(msgfst4,"0"); msgfst4_pos++; break;
          case FST_1:   strcat(msgfst4,"1"); msgfst4_pos++; break;
          case FST_2:   strcat(msgfst4,"2"); msgfst4_pos++; break;
          case FST_3:   strcat(msgfst4,"3"); msgfst4_pos++; break;
          case FST_SAVE: msgfst4[160]=0; EEPROM.put(EEPROM_MSG_FST4_ADR,msgfst4);
                        lcd.setCursor(11,0); lcd.print(F("saved"));
                        myOLED.print(F("saved"), OL_DX+11*MY_OLED_CHAR_WIDTH, OL_DY);
                        myOLED.update();
                        delay(1000);
                        lcd.setCursor(11,0); lcd.print(F("     "));
                        myOLED.print(F("saved"), OL_DX+11*MY_OLED_CHAR_WIDTH, OL_DY);
                        myOLED.update();
                        break;
          case FST_DEL: len=strlen(msgfst4);if(len>0){msgfst4[len-1]=0;msgfst4_pos--;} break;
          case FST_ERASE: memset(msgfst4,0,sizeof(msgfst4)); msgfst4_pos=0;break;
          case FST_GEN: memset(msgfst4,0,sizeof(msgfst4)); msgfst4_pos=0;
                        //bookmark1
                        clearLine(1); lcd.print(call);
                        lcd.setCursor(7,1); lcd.print(qth);
                        lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
                        myOLED.print(call,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                        myOLED.print(qth,7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                        myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                        myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
                        fst4w_init_beacon(call, qth, pwr);
                        for(int i=0; i<MSG_FST4_LEN_MAX;i++)msgfst4[i]=tx[i]-1;
                        msgfst4_pos=strlen(msgfst4);
                        break;
        }
        printFst4Menu(fst); printFst4msg();
      }
      if(mode==MODE_CALL)
      {
        call[call_pos]=dfcw_to_char(cc);
        if(call[call_pos]==0 || cc_pos==0){if(cc_pos==0)call_pos--;if(call_pos<0)call_pos=0;call[call_pos]=0;}//ошибочный ввод или стереть
        else {call_pos++; if(call_pos>=CALL_LEN_MAX)call_pos=CALL_LEN_MAX-1;call[call_pos+1]=0;}
        clearLine(0); lcd.print(F("CALL:"));
        myOLED.print(F("CALL:"), OL_DX, OL_DY);
        clearLine(1); lcd.print(call);
        myOLED.print(call, OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
        cc_pos=0; memset(cc,0,sizeof(cc));
        EEPROM.put(EEPROM_CALL_ADR,call);
      } 
      if(mode==MODE_QTH)
      {
        qth[qth_pos]=dfcw_to_char(qq);
        if(qth_pos<2) if(!isalpha(qth[qth_pos]))qth[qth_pos]=0;
        if(qth_pos==2 || qth_pos==3) if(!isdigit(qth[qth_pos]))qth[qth_pos]=0;
        if(qth[qth_pos]==0 || qq_pos==0){if(qq_pos==0)qth_pos--;if(qth_pos<0)qth_pos=0;qth[qth_pos]='_';}//ошибочный ввод или стереть
        else {qth_pos++; if(qth_pos>=QTH_LEN_MAX)qth_pos=QTH_LEN_MAX-1;qth[qth_pos+1]=0;}
        clearLine(0); lcd.setCursor(7,0); lcd.print(F("LOC:"));
        myOLED.print(F("LOC:"), 7*MY_OLED_CHAR_WIDTH+OL_DX, OL_DY);
        clearLine(1); lcd.setCursor(7,1); lcd.print(qth);
        myOLED.print(qth, 7*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
        qq_pos=0; memset(qq,0,sizeof(qq));
        EEPROM.put(EEPROM_QTH_ADR,qth);
      }
      if(mode==MODE_CLR_EEPROM && clr_eeprom)
      { byte b=0xff;
        for (int i = 0 ; i < EEPROM.length() ; i++) {EEPROM.write(i, b);} //EEPROM.length()
        clr_eeprom=0;
        clearLine(1);lcd.print(F("EEPROM is empty"));
        myOLED.print(F("EEPROM is empty"), OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_LOAD_FST4 && msgfst4_pos==MSG_FST4_LEN_MAX)
      { msgfst4[160]=0;
        EEPROM.put(EEPROM_MSG_FST4_ADR,msgfst4);
        clearLine(1);lcd.print(F("FST4 saved OK"));
        myOLED.print(F("FST4 saved OK"), OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_GEN)
      {
        Crystal=Crystal_list[list_n];
        gen_updw();
        lcd.print(F(" OK"));
        EEPROM.put(EEPROM_CRYSTAL_ADR,Crystal);
        if(DDSAD) set28bit();
        else{
         if(BIT32)set32bit(); else set24bit(); 
        }
        freq=Step*Code;
      }
      if(mode==MODE_PWR)
      {
        EEPROM.put(EEPROM_PWR_ADR,pwr);
        lcd.setCursor(9,1); lcd.print(F("OK"));
        myOLED.print(F("OK"), 9*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_QRSS)
      {
        EEPROM.put(EEPROM_DOTQRSS_ADR,dot_qrss);
        lcd.setCursor(9,1); lcd.print(F("OK"));
        myOLED.print(F("OK"), 9*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_PAUSE)
      {
        EEPROM.put(EEPROM_DOTPAUSE_ADR,dot_pause);
        lcd.setCursor(6,1); lcd.print(F("OK"));
        myOLED.print(F("OK"), 6*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_DOT_DFCW)
      {
       EEPROM.put(EEPROM_DOTDFCW_ADR,dot_dfcw);
       lcd.setCursor(9,1); lcd.print(F("OK"));
       myOLED.print(F("OK"), 9*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_DOT_HELL)
      {
       EEPROM.put(EEPROM_DOTHELL_ADR,dot_hell);
       lcd.setCursor(9,1); lcd.print(F("OK"));
       myOLED.print(F("OK"), 9*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_DOT_JT9)
      {
       EEPROM.put(EEPROM_DOTJT9_ADR,dot_jt9);
       lcd.setCursor(14,0); lcd.print(F("OK"));
       myOLED.print(F("OK"), 14*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_DOT_FST4)
      {
       EEPROM.put(EEPROM_DOTFST4_ADR,dot_fst4);
       lcd.setCursor(14,0); lcd.print(F("OK"));
       myOLED.print(F("OK"), 14*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_TIME)
      { if(timeIsSet) {timeIsSet=false;lcd.setCursor(13,1); lcd.print("* ");myOLED.print(F("* "), 13*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);myOLED.update();t_set_min=t_min; t_set_sec=t_sec;}
        else {
         setmytime();
         lcd.setCursor(13,1); lcd.print(F("OK"));
         myOLED.print(F("OK"), 13*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
         myOLED.update();
         }
      }
      if(mode==MODE_T_BEACON_START) {
        EEPROM.put(EEPROM_TBEACONSTARTHH_ADR,t_bcn_start_hh);
        EEPROM.put(EEPROM_TBEACONSTARTMM_ADR,t_bcn_start_mm);
        lcd.setCursor(15,0);lcd.print(F("*"));myOLED.print(F("*"),15*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
        myOLED.update();
      }
      if(mode==MODE_T_BEACON_STOP)  {
        EEPROM.put(EEPROM_TBEACONSTOPHH_ADR,t_bcn_stop_hh);
        EEPROM.put(EEPROM_TBEACONSTOPMM_ADR,t_bcn_stop_mm);
        lcd.setCursor(15,1);lcd.print(F("*"));myOLED.print(F("*"),15*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      if(mode==MODE_BEACON)
      {byte len=strlen(bcn_list);
        if(len>=BCN_LIST_LEN && bcn>=BCN_QRSS && bcn<=BCN_PAUSE/*BCN_JT9*/)bcn_list[BCN_LIST_LEN-1]=0; 
        switch(bcn)
        { case BCN_LOAD: EEPROM.get(EEPROM_BCNLIST_ADR,bcn_list);
                        if(bcn_list[0]==(char)0xff) memset(bcn_list,0,sizeof(bcn_list));  
                        break;
          case BCN_RUN:  BEACON=true; EEPROM.put(EEPROM_BEACON_ADR,BEACON);bcn_pos=0; start_beacon(0);  break;
          case BCN_QRSS:   strcat(bcn_list,"Q"); break;
          case BCN_DFCW:   strcat(bcn_list,"D");break;
          case BCN_HELL:   strcat(bcn_list,"H");break;
          case BCN_OP32:   strcat(bcn_list,"O");break;
          case BCN_WSPR:   strcat(bcn_list,"w");break;
          case BCN_WSPR15: strcat(bcn_list,"W");break;
          case BCN_JT9:    strcat(bcn_list,"J");break;
          //case BCN_FST4:   strcat(bcn_list,"f");break;
          case BCN_FW2:   strcat(bcn_list,"2");break;
          case BCN_FW5:   strcat(bcn_list,"5");break;
          case BCN_FW15:   strcat(bcn_list,"f");break;
          case BCN_FW30:   strcat(bcn_list,"F");break;
          case BCN_RPT:    strcat(bcn_list,"R");break;
          case BCN_T60:    strcat(bcn_list,"T");break;
          case BCN_T30:    strcat(bcn_list,"t");break;
          case BCN_TSTART:    if(bcn_list[0]!='='){
                                for(byte i=BCN_LIST_LEN;i>0;i--) bcn_list[i]=bcn_list[i-1]; 
                                bcn_list[0]='=';
                              }
                              break;
          case BCN_NO_TSTART: if(bcn_list[0]=='='){
                               byte i=0; while(bcn_list[i]&&i<BCN_LIST_LEN){bcn_list[i]=bcn_list[i+1];i++;} 
                              } 
                              break;
          case BCN_PAUSE:    strcat(bcn_list,"P");break;
          case BCN_SAVE: EEPROM.put(EEPROM_BCNLIST_ADR,bcn_list);
                         lcd.setCursor(12,0); lcd.print(F("Ok  "));
                         myOLED.print(F("Ok  "),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); myOLED.update();
                         delay(500);
                         lcd.setCursor(12,0); lcd.print(F("    "));
                         myOLED.print(F("    "),12*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY); myOLED.update();
                       break;
          case BCN_CLR: len=strlen(bcn_list);if(len>0)bcn_list[len-1]=0; break;
        }
        printBeaconList();
      }
      if(mode==MODE_SUBMENU)
      {
       submenu=!submenu; 
       if(submenu) mode++; //else mode--;
       printMode(mode); 
      }
      if(mode==MODE_LF2)
      {
       EEPROM.put(EEPROM_LF2_ADR,LF2); lcd.setCursor(0,1); lcd.print(F("Ok"));
       myOLED.print(F("Ok"), OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_BORDERS_LF)
      {
       EEPROM.put(EEPROM_BORDERS_LF_ADR,borders_lf); lcd.setCursor(0,1); lcd.print(F("Ok"));
       myOLED.print(F("Ok"), OL_DX, MY_OLED_STRING_HIGH+OL_DY);
       myOLED.update();
      }
      if(mode==MODE_TIMER_CORRECTION)
      {
        EEPROM.put(EEPROM_TIMERCOR_ADR,timer_correction); lcd.setCursor(14,1); lcd.print(F("Ok"));
        myOLED.print(F("Ok"), 14*MY_OLED_CHAR_WIDTH+OL_DX, MY_OLED_STRING_HIGH+OL_DY);
        myOLED.update();
      }
      last_btn=button;
      break;
      
    default:
    
      break;
   }
 }
  btn_old=button;
}
/////////////////////////////////////////////////////////////////////
///// одинаковые операции при нажатии кнопок UP DOWN (экономим место)
/////////////////////////////////////////////////////////////////////
void dotjt9updw(void){
  dot_jt9=jt9_list[cnt_dot_jt9];
  lcd.setCursor(11,0);
  lcd.print(dot_jt9);
  lcd.print(F("    "));
  myOLED.print(F("      "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.printNumI(dot_jt9,11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.update();
}
void dotfst4updw(void){
  dot_fst4=fst4_list[cnt_dot_fst4];
  lcd.setCursor(11,0);
  lcd.print(dot_fst4);
  lcd.print(F("    "));
  myOLED.print(F("      "),11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.printNumI(dot_fst4,11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.update();
}

void power_updw(void){
  pwr=pwr_list[n_pwr];
  if(pwr>60)pwr=60;
  clearLine(1);  lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
  myOLED.printNumI(pwr,12*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.print(F("dB"),14*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.update();
}
void clr_eeprom_updw(void){
  if(clr_eeprom){clr_eeprom=0;clearLine(1);lcd.setCursor(7,1); lcd.print(F("NO"));myOLED.print(F("NO"),7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);}
  else{clr_eeprom=1;clearLine(1);lcd.setCursor(2,1); lcd.print(F("YES [SELECT]"));myOLED.print(F("YES [SELECT]"),2*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);}
  myOLED.update();
}
void msg_updw(void){
  lcd.setCursor(11,0);
  lcd.print(m);
  myOLED.print(m,11*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.update();
  m_pos++;if(m_pos>=M_LEN_MAX-1)m_pos=M_LEN_MAX-1;
}
void call_updw(void){
  lcd.setCursor(5,0);
  lcd.print(cc);
  myOLED.print(cc,5*MY_OLED_CHAR_WIDTH+OL_DX,OL_DY);
  myOLED.update();
  cc_pos++; if(cc_pos>=CC_LEN_MAX-1)cc_pos=CC_LEN_MAX-1;
}
void msg_select(char * msg, byte &msg_pos)
{
  msg[msg_pos]=dfcw_to_char(m);
  if(msg[msg_pos]==0 || m_pos==0){if(m_pos==0)msg_pos--;if(msg_pos<0)msg_pos=0;msg[msg_pos]=0;}//ошибочный ввод или стереть
  else {msg_pos++; if(msg_pos>=MSG_LEN_MAX)msg_pos=MSG_LEN_MAX-1;msg[msg_pos+1]=0;}
  clearLine(1); lcd.print(msg);
  myOLED.print(msg,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.update();
  m_pos=0; memset(m,0,sizeof(m));
}

void gen_updw(void){
  clearLine(1);lcd.print(Crystal_list[list_n]);lcd.print(F(" Hz"));
  myOLED.printNumI(Crystal_list[list_n],OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.print(F(" Hz"),9*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.update();
}
void lf2updw(void){
  clearLine(0); 
  if(LF2){lcd.print(F("Fout=2*F(272kHz)")); myOLED.print(F("Fout=2*F(272kHz)"),CENTER,OL_DY);}
  else {lcd.print(F("Fout=F (136kHz)")); myOLED.print(F("Fout=F (136kHz)"),CENTER,OL_DY);}
  myOLED.update();
}
void borderslfupdw(void){
  clearLine(0); 
  if(borders_lf){lcd.print(F("FREQ in LF")); myOLED.print(F("FREQ in LF"),CENTER,OL_DY);}
  else {lcd.print(F("FREQ is FREE"));myOLED.print(F("FREQ is FREE"),CENTER,OL_DY);}
  myOLED.update();
}
void updwdc(byte mod, byte bt)
{long dc1=0L;
 if(last_btn==bt) {updwcnt++;if(updwcnt<10) dc=1;} 
 else {updwcnt=0; dc=1;}
 if(updwcnt>2 && double(dc*Step)<0.01)  dc=(unsigned long)(0.01/Step)+1;
 if(updwcnt>10) dc=(unsigned long)(0.1/Step)+1;
 if(updwcnt>20) dc=(unsigned long)(0.5/Step);
 if(updwcnt>40) dc=(unsigned long)(1.0/Step);
 if(updwcnt>80) dc=(unsigned long)(5.0/Step);
 if(!borders_lf){
  if(updwcnt>80) dc=(unsigned long)(1000.0/Step);
  if(updwcnt>100) dc=(unsigned long)(10000.0/Step);
  if(updwcnt>120) dc=(unsigned long)(100000.0/Step);
  //if(updwcnt>140) dc=(unsigned long)(1000000.0/Step);
 }
 
 if(bt==BTN_UP)   dc1=dc;
 if(bt==BTN_DOWN) {if(dc*Step > freq/4) dc=freq/4 + 1; dc1=-dc; }

 if(mod==MODE_CTRL) { CodeCW=CodeCW+dc1;freqCW=CodeCW*Step;
                      if(borders_lf){
                        if(freqCW<135700.0)CodeCW=(unsigned long)(135700.1/Step);
                        if(freqCW>137800.0)CodeCW=(unsigned long)(137799.9/Step);
                      }
                      printFreq(CodeCW);
                     } 
 if(mod==MODE_QRSS) {CodeQRSS=CodeQRSS+dc1; freq=CodeQRSS*Step;
                      if(borders_lf){
                      if(bt==BTN_UP){ 
                       if(freq>136180.0 && freq<137700.0) {CodeQRSS=(unsigned long)(137700.0/Step); if(dot_qrss>=60)CodeQRSS=(unsigned long)(137774.0/Step);updwcnt=0;}
                       if(freq>137783.0)CodeQRSS=(unsigned long)(137783.0/Step);//(136168.0/Step);
                      }
                      else{
                       if(freq>136180.0 && freq<137700.0) {CodeQRSS=(unsigned long)(136180.0/Step);updwcnt=0;} 
                       if(freq<136168.0)CodeQRSS=(unsigned long)(136168.0/Step);//(137783.0/Step);
                      }
                      }
                      printFreq(CodeQRSS);
                     }
 if(mod==MODE_DFCW) {CodeDFCW=CodeDFCW+dc1; freq=CodeDFCW*Step; 
                      if(borders_lf){
                      if(bt==BTN_UP){
                       if(freq>136180.0 && freq<137700.0) {CodeDFCW=(unsigned long)(137700.0/Step);if(dot_dfcw>=60)CodeDFCW=(unsigned long)(137774.0/Step);updwcnt=0;}
                       if(freq>137783.0) CodeDFCW=(unsigned long)(137783.0/Step);//(136168.0/Step);
                      }
                      else {
                       if(freq>136180.0 && freq<137700.0) {CodeDFCW=(unsigned long)(136180.0/Step);updwcnt=0;} 
                       if(freq<136168.0) CodeDFCW=(unsigned long)(136168.0/Step);//(137783.0/Step);
                      }
                      }
                      printFreq(CodeDFCW);
                     }
 if(mod==MODE_HELL) {CodeHELL=CodeHELL+dc1; freq=CodeHELL*Step; 
                      if(borders_lf){
                      if(bt==BTN_UP){
                       if(freq>136180.0 && freq<137700.0) {CodeHELL=(unsigned long)(137700.0/Step);if(dot_hell>=60)CodeHELL=(unsigned long)(137774.0/Step);updwcnt=0;}
                       if(freq>137783.0) CodeHELL=(unsigned long)(137783.0/Step);//(136168.0/Step);
                      }
                      else {
                       if(freq>136180.0 && freq<137700.0) {CodeHELL=(unsigned long)(136180.0/Step);updwcnt=0;} 
                       if(freq<136168.0) CodeHELL=(unsigned long)(136168.0/Step);//(137783.0/Step);
                      }
                      }
                      printFreq(CodeHELL);
                     }

 if(mod==MODE_WSPR) {CodeWSPR=CodeWSPR+dc1;
                      if(borders_lf){ 
                       if(CodeWSPR*Step>137600.0) CodeWSPR=(unsigned long)(137600.0/Step);
                       if(CodeWSPR*Step<137400.0) CodeWSPR=(unsigned long)(137400.0/Step);
                      }
                      printFreq(CodeWSPR);
                     }
 if(mod==MODE_WSPR15) {CodeWSPR15=CodeWSPR15+dc1;
                      if(borders_lf){ 
                        if(CodeWSPR15*Step>137625.0) CodeWSPR15=(unsigned long)(137625.0/Step);
                        if(CodeWSPR15*Step<137600.0) CodeWSPR15=(unsigned long)(137600.0/Step);
                      }
                        printFreq(CodeWSPR15);
                       }
 if(mod==MODE_JT9) {CodeJT9=CodeJT9+dc1;
                      if(borders_lf){ 
                       if(CodeJT9*Step>137400.0) CodeJT9=(unsigned long)(137400.0/Step);
                       if(CodeJT9*Step<137000.0) CodeJT9=(unsigned long)(137000.0/Step);
                      }
                      printFreq(CodeJT9);
                     }
 if(mod==MODE_FST4||mod==MODE_FW) {CodeFST4=CodeFST4+dc1;
                      if(borders_lf){ 
                       if(CodeFST4*Step>137700.0) CodeFST4=(unsigned long)(137700.0/Step);
                       if(CodeFST4*Step<137000.0) CodeFST4=(unsigned long)(137000.0/Step);
                      }
                      printFreq(CodeFST4);
                     }
 if(mod==MODE_OP32) { CodeOP32=CodeOP32+dc1;
                      if(borders_lf){ 
                       if(CodeOP32*Step>137550.0) CodeOP32=(unsigned long)(137550.0/Step);
                       if(CodeOP32*Step<137450.0) CodeOP32=(unsigned long)(137450.0/Step);
                      }
                       printFreq(CodeOP32);
                     }
  myOLED.update();

}

void timcorupdw(void){
  clearLine(1);
  lcd.print(0.6*timer_correction); lcd.print(F(" s"));
  myOLED.printNumF(0.6*timer_correction,3,OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.print(F(" s"),7*MY_OLED_CHAR_WIDTH+OL_DX,MY_OLED_STRING_HIGH+OL_DY);
  myOLED.update();
}
void compute_time(void){
  uint32_t sec = (millis() + t_dif + (millis()-time_of_settime)*timer_correction/1000.0) / 1000UL;
  //int timeHours = (sec / 3600ul) % 24;
  t_hour= (byte)((sec / 3600UL) % 24UL);
  t_min = (byte)((sec % 3600UL) / 60UL);
  t_sec = (byte)((sec % 3600UL) % 60UL);
}
void setmytime(void){
  int dh=t_set_hour-t_hour;
  int dm=t_set_min-t_min;
  int ds=t_set_sec-t_sec;
  if(ds<0) {ds=ds+60;dm=dm-1;}
  if(dm<0) {dm=dm+60;dh=dh-1;}
  if(dh<0) {dh=dh+24;}
  t_dif+=(dh*3600UL + dm*60UL + ds)*1000UL;
  timeIsSet=true;
  time_of_settime=millis();
  rtc.adjust(DateTime(2020, 10, 8, t_set_hour, t_set_min, t_set_sec)); 
}
